searchNodes=[{"doc":"Use Commanded to build your own Elixir applications following the CQRS/ES pattern. Provides support for: Defining applications . Command registration and dispatch . Hosting and delegation to aggregates . Event handling . Long running process managers . Use Commanded with one of the following event stores for persistence: Elixir EventStore using PostgreSQL for persistence EventStore.org Please check the Getting Started and Usage guides to learn more.","ref":"Commanded.html","title":"Commanded","type":"module"},{"doc":"Retrieve aggregate state of an aggregate. Retrieving aggregate state is done by calling to the opened aggregate, or querying the event store for an optional state snapshot and then replaying the aggregate's event stream.","ref":"Commanded.html#aggregate_state/4","title":"Commanded.aggregate_state/4","type":"function"},{"doc":"Defines a Commanded application. The application expects at least an :otp_app option to be specified. It should point to an OTP application that has the application configuration. For example, the application: defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app router ( MyApp.Router ) end Could be configured with: # config/config.exs config :my_app , MyApp.Application event_store : [ adapter : Commanded.EventStore.Adapters.EventStore , event_store : MyApp.EventStore ] , pubsub : :local , registry : :local Alternatively, you can include the configuration when defining the application: defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app , event_store : [ adapter : Commanded.EventStore.Adapters.EventStore , event_store : MyApp.EventStore ] , pubsub : :local , registry : :local router ( MyApp.Router ) end A Commanded application must be started before it can be used: { :ok , _pid } = MyApp.Application . start_link ( ) Instead of starting the application manually, you should use a Supervisor . Supervision Use a supervisor to start your Commanded application: Supervisor . start_link ( [ MyApp.Application ] , strategy : :one_for_one ) Command routing Commanded applications are also composite routers allowing you to include one or more routers within an application. Example defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app router ( MyApp.Accounts.Router ) router ( MyApp.Billing.Router ) router ( MyApp.Notifications.Router ) end See Commanded.Commands.CompositeRouter for details. Command dispatch Once a router has been configured you can dispatch a command via the application: :ok = MyApp . dispatch ( command , opts ) See dispatch/1 and dispatch/2 for details. Dynamic named applications An application can be provided with a name as an option to start_link/1 . This can be used to start the same application multiple times, each using its own separately configured and isolated event store. Each application must be started with a unique name. Multiple instances of the same event handler or process manager can be started by referring to a started application by its name. The event store operations can also be scoped to an application by referring to its name. Example Start an application process for each tenant in a multi-tenanted app, guaranteeing that the data and processing remains isolated between tenants. for tenant &lt;- [ :tenant1 , :tenant2 , :tenant3 ] do { :ok , _app } = MyApp.Application . start_link ( name : tenant ) end Typically you would start the applications using a supervisor: children = for tenant &lt;- [ :tenant1 , :tenant2 , :tenant3 ] do { MyApp.Application , name : tenant } end Supervisor . start_link ( children , strategy : :one_for_one ) To dispatch a command you must provide the application name: :ok = MyApp.Application . dispatch ( command , application : :tenant1 ) Default dispatch options An application can be configured with default command dispatch options such as :consistency , :timeout , and :returning . Any defaults will be used unless overridden by options provided to the dispatch function. defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app , default_dispatch_opts : [ consistency : :eventual , returning : :aggregate_version ] end See the Commanded.Commands.Router module for more details about the supported options. Telemetry [:commanded, :application, :dispatch, :start] Description: Emitted when an application starts dispatching a command Measurements: %{system_time: integer()} Metadata: %{application: Commanded.Application.t(), execution_context: Commanded.Aggregates.ExecutionContext.t()} [:commanded, :application, :dispatch, :stop] Description: Emitted when an application stops dispatching a command Measurements: %{duration: non_neg_integer()} Metadata: %{application: Commanded.Application.t(), execution_context: Commanded.Aggregates.ExecutionContext.t(), error: nil | any()}","ref":"Commanded.Application.html","title":"Commanded.Application","type":"behaviour"},{"doc":"Returns the application configuration stored in the :otp_app environment.","ref":"Commanded.Application.html#c:config/0","title":"Commanded.Application.config/0","type":"callback"},{"doc":"Dispatch a registered command. command is a command struct which must be registered with a Commanded.Commands.Router and included in the application.","ref":"Commanded.Application.html#c:dispatch/1","title":"Commanded.Application.dispatch/1","type":"callback"},{"doc":"Dispatch a registered command. command is a command struct which must be registered with a Commanded.Commands.Router and included in the application. timeout_or_opts is either an integer timeout or a keyword list of options. The timeout must be an integer greater than zero which specifies how many milliseconds to allow the command to be handled, or the atom :infinity to wait indefinitely. The default timeout value is five seconds. Alternatively, an options keyword list can be provided, it supports the following options. Options: causation_id - an optional UUID used to identify the cause of the command being dispatched. correlation_id - an optional UUID used to correlate related commands/events together. consistency - one of :eventual (default) or :strong . By setting the consistency to :strong a successful command dispatch will block until all strongly consistent event handlers and process managers have handled all events created by the command. metadata - an optional map containing key/value pairs comprising the metadata to be associated with all events created by the command. returning - to choose what response is returned from a successful command dispatch. The default is to return an :ok . The available options are: :aggregate_state - to return the update aggregate state in the successful response: {:ok, aggregate_state} . :aggregate_version - to include the aggregate stream version in the successful response: {:ok, aggregate_version} . :execution_result - to return a Commanded.Commands.ExecutionResult struct containing the aggregate's identity, version, and any events produced from the command along with their associated metadata. false - don't return anything except an :ok . timeout - as described above. Returns :ok on success unless the :returning option is specified where it returns one of {:ok, aggregate_state} , {:ok, aggregate_version} , or {:ok, %Commanded.Commands.ExecutionResult{}} . Returns {:error, reason} on failure. Example command = % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } :ok = BankApp . dispatch ( command , timeout : 30_000 )","ref":"Commanded.Application.html#c:dispatch/2","title":"Commanded.Application.dispatch/2","type":"callback"},{"doc":"A callback executed when the application starts. It must return {:ok, keyword} with the updated list of configuration.","ref":"Commanded.Application.html#c:init/1","title":"Commanded.Application.init/1","type":"callback"},{"doc":"Starts the application supervisor. Returns {:ok, pid} on success, {:error, {:already_started, pid}} if the application is already started, or {:error, term} in case anything else goes wrong.","ref":"Commanded.Application.html#c:start_link/1","title":"Commanded.Application.start_link/1","type":"callback"},{"doc":"Shuts down the application.","ref":"Commanded.Application.html#c:stop/2","title":"Commanded.Application.stop/2","type":"callback"},{"doc":"","ref":"Commanded.Application.html#t:t/0","title":"Commanded.Application.t/0","type":"type"},{"doc":"Use Commanded.Aggregate.Multi to generate multiple events from a single command. This can be useful when you want to emit multiple events that depend upon the aggregate state being updated. Example In the example below, money is withdrawn from the bank account and the updated balance is used to check whether the account is overdrawn. defmodule BankAccount do alias Commanded.Aggregate.Multi defstruct [ :account_number , :state , balance : 0 ] def withdraw ( % BankAccount { state : :active } = account , % WithdrawMoney { amount : amount } ) when is_number ( amount ) and amount &gt; 0 do account |&gt; Multi . new ( ) |&gt; Multi . execute ( &amp; withdraw_money ( &amp;1 , amount ) ) |&gt; Multi . execute ( &amp; check_balance / 1 ) end defp withdraw_money ( % BankAccount { account_number : account_number , balance : balance } , amount ) do % MoneyWithdrawn { account_number : account_number , amount : amount , balance : balance - amount } end defp check_balance ( % BankAccount { account_number : account_number , balance : balance } ) when balance &lt; 0 do % AccountOverdrawn { account_number : account_number , balance : balance } end defp check_balance ( % BankAccount { } ) , do : [ ] end","ref":"Commanded.Aggregate.Multi.html","title":"Commanded.Aggregate.Multi","type":"module"},{"doc":"Adds a command execute function to the multi.","ref":"Commanded.Aggregate.Multi.html#execute/2","title":"Commanded.Aggregate.Multi.execute/2","type":"function"},{"doc":"Create a new Commanded.Aggregate.Multi struct.","ref":"Commanded.Aggregate.Multi.html#new/1","title":"Commanded.Aggregate.Multi.new/1","type":"function"},{"doc":"Reduce an enumerable by executing the function for each item. The aggregate apply/2 function will be called after each event returned by the execute function. This allows you to calculate values from the aggregate state based upon events produced by previous items in the enumerable, such as running totals. Example alias Commanded.Aggregate.Multi aggregate |&gt; Multi . new ( ) |&gt; Multi . reduce ( [ 1 , 2 , 3 ] , fn aggregate , item -&gt; % AnEvent { item : item , total : aggregate . total + item } end )","ref":"Commanded.Aggregate.Multi.html#reduce/3","title":"Commanded.Aggregate.Multi.reduce/3","type":"function"},{"doc":"Run the execute functions contained within the multi, returning the updated aggregate state and all created events.","ref":"Commanded.Aggregate.Multi.html#run/1","title":"Commanded.Aggregate.Multi.run/1","type":"function"},{"doc":"","ref":"Commanded.Aggregate.Multi.html#t:t/0","title":"Commanded.Aggregate.Multi.t/0","type":"type"},{"doc":"Aggregate is a GenServer process used to provide access to an instance of an event sourced aggregate. It allows execution of commands against an aggregate instance, and handles persistence of created events to the configured event store. Concurrent commands sent to an aggregate instance are serialized and executed in the order received. The Commanded.Commands.Router module will locate, or start, an aggregate instance when a command is dispatched. By default, an aggregate process will run indefinitely once started. Its lifespan may be controlled by using the Commanded.Aggregates.AggregateLifespan behaviour. Snapshotting You can configure state snapshots for an aggregate in config. By default snapshots are not taken for an aggregate. The following options are available to enable snapshots: snapshot_every - snapshot aggregate state every so many events. Use nil to disable snapshotting, or exclude the configuration entirely. snapshot_version - a non-negative integer indicating the version of the aggregate state snapshot. Incrementing this version forces any earlier recorded snapshots to be ignored when rebuilding aggregate state. Example In config/config.exs enable snapshots for MyApp.ExampleAggregate after every ten events: config :my_app , MyApp.Application , snapshotting : %{ MyApp.ExampleAggregate =&gt; [ snapshot_every : 10 , snapshot_version : 1 ] } Telemetry [:commanded, :aggregate, :execute, :start] Description: Emitted when an aggregate starts executing a command Measurements: %{system_time: integer()} Metadata: %{application: Commanded.Application.t(), aggregate_uuid: String.t(), aggregate_state: struct(), aggregate_version: non_neg_integer(), caller: pid(), execution_context: Commanded.Aggregates.ExecutionContext.t()} [:commanded, :aggregate, :execute, :stop] Description: Emitted when an aggregate stops executing a command Measurements: %{duration: non_neg_integer()} Metadata: %{application: Commanded.Application.t(), aggregate_uuid: String.t(), aggregate_state: struct(), aggregate_version: non_neg_integer(), caller: pid(), execution_context: Commanded.Aggregates.ExecutionContext.t(), events: [map()], error: nil | any()} [:commanded, :aggregate, :execute, :exception] Description: Emitted when an aggregate raises an exception Measurements: %{duration: non_neg_integer()} Metadata: %{application: Commanded.Application.t(), aggregate_uuid: String.t(), aggregate_state: struct(), aggregate_version: non_neg_integer(), caller: pid(), execution_context: Commanded.Aggregates.ExecutionContext.t(), kind: :throw | :error | :exit, reason: any(), stacktrace: list()}","ref":"Commanded.Aggregates.Aggregate.html","title":"Commanded.Aggregates.Aggregate","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Commanded.Aggregates.Aggregate.html#child_spec/1","title":"Commanded.Aggregates.Aggregate.child_spec/1","type":"function"},{"doc":"Execute the given command against the aggregate. aggregate_module - the aggregate's module (e.g. BankAccount ). aggregate_uuid - uniquely identifies an instance of the aggregate. context - includes command execution arguments (see Commanded.Aggregates.ExecutionContext for details). timeout - an non-negative integer which specifies how many milliseconds to wait for a reply, or the atom :infinity to wait indefinitely. The default value is five seconds (5,000ms). Return values Returns {:ok, aggregate_version, events} on success, or {:error, error} on failure. aggregate_version - the updated version of the aggregate after executing the command. events - events produced by the command, can be an empty list.","ref":"Commanded.Aggregates.Aggregate.html#execute/5","title":"Commanded.Aggregates.Aggregate.execute/5","type":"function"},{"doc":"","ref":"Commanded.Aggregates.Aggregate.html#start_link/2","title":"Commanded.Aggregates.Aggregate.start_link/2","type":"function"},{"doc":"The Commanded.Aggregates.AggregateLifespan behaviour is used to control the aggregate GenServer process lifespan. By default an aggregate instance process will run indefinitely once started. You can change this default by implementing the Commanded.Aggregates.AggregateLifespan behaviour in a module and configuring it in your router. After a command successfully executes, and creates at least one domain event, the after_event/1 function is called passing the last created event. When a command is successfully handled but results in no domain events (by returning nil or an empty list [] ), the command struct is passed to the after_command/1 function. Finally, if there is an error executing the command, the error reason is passed to the after_error/1 function. For all the above, the returned inactivity timeout value is used to shutdown the aggregate process if no other messages are received. Supported return values Non-negative integer - specify an inactivity timeout, in millisconds. :infinity - prevent the aggregate instance from shutting down. :hibernate - send the process into hibernation. :stop - immediately shutdown the aggregate process with a :normal exit reason. {:stop, reason} - immediately shutdown the aggregate process with the given reason. Hibernation A hibernated process will continue its loop once a message is in its message queue. Hibernating an aggregate causes garbage collection and minimises the memory used by the process. Hibernating should not be used aggressively as too much time could be spent garbage collecting. Example Define a module that implements the Commanded.Aggregates.AggregateLifespan behaviour: defmodule BankAccountLifespan do @behaviour Commanded.Aggregates.AggregateLifespan def after_event ( % MoneyDeposited { } ) , do : :timer . hours ( 1 ) def after_event ( % BankAccountClosed { } ) , do : :stop def after_event ( _event ) , do : :infinity def after_command ( % CloseAccount { } ) , do : :stop def after_command ( _command ) , do : :infinity def after_error ( :invalid_initial_balance ) , do : :timer . minutes ( 5 ) def after_error ( _error ) , do : :stop end Then specify the module as the lifespan option when registering the applicable commands in your router: defmodule BankRouter do use Commanded.Commands.Router dispatch [ OpenAccount , CloseAccount ] , to : BankAccount , lifespan : BankAccountLifespan , identity : :account_number end","ref":"Commanded.Aggregates.AggregateLifespan.html","title":"Commanded.Aggregates.AggregateLifespan","type":"behaviour"},{"doc":"Aggregate process will be stopped after specified inactivity timeout unless :infinity , :hibernate , or :stop are returned.","ref":"Commanded.Aggregates.AggregateLifespan.html#c:after_command/1","title":"Commanded.Aggregates.AggregateLifespan.after_command/1","type":"callback"},{"doc":"Aggregate process will be stopped after specified inactivity timeout unless :infinity , :hibernate , or :stop are returned.","ref":"Commanded.Aggregates.AggregateLifespan.html#c:after_error/1","title":"Commanded.Aggregates.AggregateLifespan.after_error/1","type":"callback"},{"doc":"Aggregate process will be stopped after specified inactivity timeout unless :infinity , :hibernate , or :stop are returned.","ref":"Commanded.Aggregates.AggregateLifespan.html#c:after_event/1","title":"Commanded.Aggregates.AggregateLifespan.after_event/1","type":"callback"},{"doc":"","ref":"Commanded.Aggregates.AggregateLifespan.html#t:lifespan/0","title":"Commanded.Aggregates.AggregateLifespan.lifespan/0","type":"type"},{"doc":"","ref":"Commanded.Aggregates.AggregateStateBuilder.html","title":"Commanded.Aggregates.AggregateStateBuilder","type":"module"},{"doc":"Populate the aggregate's state from a snapshot, if present, and it's events. Attempt to fetch a snapshot for the aggregate to use as its initial state. If the snapshot exists, fetch any subsequent events to rebuild its state. Otherwise start with the aggregate struct and stream all existing events for the aggregate from the event store to rebuild its state from those events.","ref":"Commanded.Aggregates.AggregateStateBuilder.html#populate/1","title":"Commanded.Aggregates.AggregateStateBuilder.populate/1","type":"function"},{"doc":"Load events from the event store, in batches, to rebuild the aggregate state","ref":"Commanded.Aggregates.AggregateStateBuilder.html#rebuild_from_events/1","title":"Commanded.Aggregates.AggregateStateBuilder.rebuild_from_events/1","type":"function"},{"doc":"The default implementation of the Commanded.Aggregates.AggregateLifespan behaviour. It will ensure that an aggregate instance process runs indefinitely once started, unless an exception is encountered.","ref":"Commanded.Aggregates.DefaultLifespan.html","title":"Commanded.Aggregates.DefaultLifespan","type":"module"},{"doc":"Aggregate will run indefinitely once started.","ref":"Commanded.Aggregates.DefaultLifespan.html#after_command/1","title":"Commanded.Aggregates.DefaultLifespan.after_command/1","type":"function"},{"doc":"Aggregate is stopped on exception, but will run indefinitely for any non- exception error.","ref":"Commanded.Aggregates.DefaultLifespan.html#after_error/1","title":"Commanded.Aggregates.DefaultLifespan.after_error/1","type":"function"},{"doc":"Aggregate will run indefinitely once started.","ref":"Commanded.Aggregates.DefaultLifespan.html#after_event/1","title":"Commanded.Aggregates.DefaultLifespan.after_event/1","type":"function"},{"doc":"Defines the arguments used to execute a command for an aggregate. The available options are: command - the command to execute, typically a struct (e.g. %OpenBankAccount{...} ). retry_attempts - the number of retries permitted if an {:error, :wrong_expected_version} is encountered when appending events. causation_id - the UUID assigned to the dispatched command. correlation_id - a UUID used to correlate related commands/events. metadata - a map of key/value pairs containing the metadata to be associated with all events created by the command. handler - the module that handles the command. It may be either the aggregate module itself or a separate command handler module. function - the name of the function, as an atom, that handles the command. The default value is :execute , used to support command dispatch directly to the aggregate module. For command handlers the :handle function is used. before_execute - the name of the function, as an atom, that prepares the command before execution, called just before function . The default value is nil , disabling it. It should return :ok on success or {:error, any()} to cancel the dispatch. lifespan - a module implementing the Commanded.Aggregates.AggregateLifespan behaviour to control the aggregate instance process lifespan. The default value, Commanded.Aggregates.DefaultLifespan , keeps the process running indefinitely.","ref":"Commanded.Aggregates.ExecutionContext.html","title":"Commanded.Aggregates.ExecutionContext","type":"module"},{"doc":"","ref":"Commanded.Aggregates.ExecutionContext.html#format_reply/3","title":"Commanded.Aggregates.ExecutionContext.format_reply/3","type":"function"},{"doc":"","ref":"Commanded.Aggregates.ExecutionContext.html#retry/1","title":"Commanded.Aggregates.ExecutionContext.retry/1","type":"function"},{"doc":"Supervises Commanded.Aggregates.Aggregate instance processes.","ref":"Commanded.Aggregates.Supervisor.html","title":"Commanded.Aggregates.Supervisor","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Commanded.Aggregates.Supervisor.html#child_spec/1","title":"Commanded.Aggregates.Supervisor.child_spec/1","type":"function"},{"doc":"Callback implementation for DynamicSupervisor.init/1 .","ref":"Commanded.Aggregates.Supervisor.html#init/1","title":"Commanded.Aggregates.Supervisor.init/1","type":"function"},{"doc":"Open an aggregate instance process for the given aggregate module and unique identity. Returns {:ok, aggregate_uuid} when a process is successfully started, or is already running.","ref":"Commanded.Aggregates.Supervisor.html#open_aggregate/3","title":"Commanded.Aggregates.Supervisor.open_aggregate/3","type":"function"},{"doc":"","ref":"Commanded.Aggregates.Supervisor.html#start_link/1","title":"Commanded.Aggregates.Supervisor.start_link/1","type":"function"},{"doc":"Composite router allows you to combine multiple router modules into a single router able to dispatch any registered command from an included child router. One example usage is to define a router per context and then combine each context's router into a single top-level composite app router used for all command dispatching. Example Define a composite router module which imports the commands from each included router: defmodule Bank.AppRouter do use Commanded.Commands.CompositeRouter router ( Bank.Accounts.Router ) router ( Bank.MoneyTransfer.Router ) end One or more routers or composite routers can be included in a Commanded.Application since it is also a composite router: defmodule BankApp do use Commanded.Application router ( Bank.AppRouter ) end You can dispatch a command via the application which will then be routed to the associated child router: command = % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } :ok = BankApp . dispatch ( command ) Or via the composite router itself, specifying the application: :ok = Bank.AppRouter . dispatch ( command , application : BankApp ) A composite router can include composite routers.","ref":"Commanded.Commands.CompositeRouter.html","title":"Commanded.Commands.CompositeRouter","type":"module"},{"doc":"Register a Commanded.Commands.Router module within this composite router. Will allow the composite router to dispatch any commands registered by the included router module. Multiple routers can be registered.","ref":"Commanded.Commands.CompositeRouter.html#router/1","title":"Commanded.Commands.CompositeRouter.router/1","type":"macro"},{"doc":"Contains the aggregate, events, and metadata created by a successfully executed command. The available fields are: aggregate_uuid - identity of the aggregate instance. aggregate_state - resultant state of the aggregate after executing the command. aggregate_version - resultant version of the aggregate after executing the command. events - a list of the created events, it may be an empty list. metadata - an map containing the metadata associated with the command dispatch.","ref":"Commanded.Commands.ExecutionResult.html","title":"Commanded.Commands.ExecutionResult","type":"module"},{"doc":"","ref":"Commanded.Commands.ExecutionResult.html#t:t/0","title":"Commanded.Commands.ExecutionResult.t/0","type":"type"},{"doc":"Defines the behaviour a command handler module must implement to support command dispatch. Example An open account handler that delegates to a bank account aggregate: defmodule OpenAccountHandler do @behaviour Commanded.Commands.Handler def handle ( % BankAccount { } = aggregate , % OpenAccount { } = command ) do % OpenAccount { account_number : account_number , initial_balance : initial_balance } = command BankAccount . open_account ( aggregate , account_number , initial_balance ) end end","ref":"Commanded.Commands.Handler.html","title":"Commanded.Commands.Handler","type":"behaviour"},{"doc":"Apply the given command to the event sourced aggregate. You must return a single domain event, a list containing the pending events, or nil , [] , or :ok when no events are produced. You should return {:error, reason} on failure.","ref":"Commanded.Commands.Handler.html#c:handle/2","title":"Commanded.Commands.Handler.handle/2","type":"callback"},{"doc":"","ref":"Commanded.Commands.Handler.html#t:aggregate/0","title":"Commanded.Commands.Handler.aggregate/0","type":"type"},{"doc":"","ref":"Commanded.Commands.Handler.html#t:command/0","title":"Commanded.Commands.Handler.command/0","type":"type"},{"doc":"","ref":"Commanded.Commands.Handler.html#t:domain_event/0","title":"Commanded.Commands.Handler.domain_event/0","type":"type"},{"doc":"","ref":"Commanded.Commands.Handler.html#t:reason/0","title":"Commanded.Commands.Handler.reason/0","type":"type"},{"doc":"Command routing macro to allow configuration of each command to its command handler. Example Define a router module which uses Commanded.Commands.Router and configures available commands to dispatch: defmodule BankRouter do use Commanded.Commands.Router dispatch OpenAccount , to : OpenAccountHandler , aggregate : BankAccount , identity : :account_number end The to option determines which module receives the command being dispatched. This command handler module must implement a handle/2 function. It receives the aggregate's state and the command to execute. Usually the command handler module will forward the command to the aggregate. Once configured, you can either dispatch a command by using the module and specifying the application: command = % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } :ok = BankRouter . dispatch ( command , application : BankApp ) Or, more simply, you should include the router module in your application: defmodule BankApp do use Commanded.Application , otp_app : :my_app router MyApp.Router end Then dispatch commands using the app: command = % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } :ok = BankApp . dispatch ( command ) Dispatch command directly to an aggregate You can route a command directly to an aggregate, without requiring an intermediate command handler. Example defmodule BankRouter do use Commanded.Commands.Router # Will route to `BankAccount.open_account/2` dispatch OpenAccount , to : BankAccount , identity : :account_number end By default, you must define an execute/2 function on the aggregate module, which will be called with the aggregate's state and the command to execute. Using this approach, you must create an execute/2 clause that pattern-matches on each command that the aggregate should handle. Alternatively, you may specify the name of a function (also receiving both the aggregate state and the command) on your aggregate module to which the command will be dispatched: Example defmodule BankRouter do use Commanded.Commands.Router # Will route to `BankAccount.open_account/2` dispatch OpenAccount , to : BankAccount , function : :open_account , identity : :account_number end Define aggregate identity You can define the identity field for an aggregate once using the identify macro. The configured identity will be used for all commands registered to the aggregate, unless overridden by a command registration. Example defmodule BankRouter do use Commanded.Commands.Router identify BankAccount , by : :account_number , prefix : &quot;bank-account-&quot; dispatch OpenAccount , to : BankAccount end An optional identity prefix can be used to distinguish between different aggregates that would otherwise share the same identity. As an example you might have a User and a UserPreferences aggregate that you wish to share the same identity. In this scenario you should specify a prefix for each aggregate (e.g. &quot;user-&quot; and &quot;user-preference-&quot;). The prefix is used as the stream identity when appending and reading the aggregate's events: &quot;&lt;identity_prefix&gt;&lt;aggregate_uuid&gt;&quot;. It can be a string or a zero arity function returning a string. Consistency You can choose the consistency guarantee when dispatching a command. The available options are: :eventual (default) - don't block command dispatch while waiting for event handlers :ok = BankApp.dispatch(command) :ok = BankApp.dispatch(command, consistency: :eventual) :strong - block command dispatch until all strongly consistent event handlers and process managers have successfully processed all events created by the command. Use this when you have event handlers that update read models you need to query immediately after dispatching the command. :ok = BankApp.dispatch(command, consistency: :strong) Provide an explicit list of event handler and process manager modules (or their configured names), containing only those handlers you'd like to wait for. No other handlers will be awaited on, regardless of their own configured consistency setting. :ok = BankApp . dispatch ( command , consistency : [ ExampleHandler , AnotherHandler ] ) :ok = BankApp . dispatch ( command , consistency : [ &quot;ExampleHandler&quot; , &quot;AnotherHandler&quot; ] ) Note you cannot opt-in to strong consistency for a handler that has been configured as eventually consistent. Dispatch return By default a successful command dispatch will return :ok . You can change this behaviour by specifying a returning option. The supported options are: :aggregate_state - to return the update aggregate state. :aggregate_version - to return only the aggregate version. :events - to return the resultant domain events. An empty list will be returned if no events were produced. :execution_result - to return a Commanded.Commands.ExecutionResult struct containing the aggregate's identity, state, version, and any events produced from the command along with their associated metadata. false - don't return anything except an :ok . Aggregate state Return the updated aggregate state as part of the dispatch result: { :ok , % BankAccount { } } = BankApp . dispatch ( command , returning : :aggregate_state ) This is useful when you want to immediately return fields from the aggregate's state without requiring an read model projection and waiting for the event(s) to be projected. It may also be appropriate to use this feature for unit tests. However, be warned that tightly coupling an aggregate's state with read requests may be harmful. It's why CQRS enforces the separation of reads from writes by defining two separate and specialised models. Aggregate version You can optionally choose to return the aggregate's version as part of the dispatch result: { :ok , aggregate_version } = BankApp . dispatch ( command , returning : :aggregate_version ) This is useful when you need to wait for an event handler, such as a read model projection, to be up-to-date before querying its data. Execution results You can also choose to return the execution result as part of the dispatch result: alias Commanded.Commands.ExecutionResult { :ok , % ExecutionResult { } = result } = BankApp . dispatch ( command , returning : :execution_result ) Or by setting the default_dispatch_return in your application config file: # config/config.exs config :commanded , default_dispatch_return : :execution_result Use the execution result struct to get information from the events produced from the command. Metadata You can associate metadata with all events created by the command. Supply a map containing key/value pairs comprising the metadata: :ok = BankApp . dispatch ( command , metadata : %{ &quot;ip_address&quot; =&gt; &quot;127.0.0.1&quot; } )","ref":"Commanded.Commands.Router.html","title":"Commanded.Commands.Router","type":"behaviour"},{"doc":"Dispatch the given command to the registered handler. Returns :ok on success, or {:error, reason} on failure. Example command = % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } :ok = BankRouter . dispatch ( command )","ref":"Commanded.Commands.Router.html#c:dispatch/1","title":"Commanded.Commands.Router.dispatch/1","type":"callback"},{"doc":"Configure the command, or list of commands, to be dispatched to the corresponding handler and aggregate. Example defmodule BankRouter do use Commanded.Commands.Router dispatch [ OpenAccount , DepositMoney ] , to : BankAccount , identity : :account_number end","ref":"Commanded.Commands.Router.html#dispatch/2","title":"Commanded.Commands.Router.dispatch/2","type":"macro"},{"doc":"Dispatch the given command to the registered handler providing a timeout. command is a command struct which must be registered with the router. timeout_or_opts is either an integer timeout, :infinity , or a keyword list of options. The timeout must be an integer greater than zero which specifies how many milliseconds to allow the command to be handled, or the atom :infinity to wait indefinitely. The default timeout value is five seconds. Alternatively, an options keyword list can be provided with the following options. Options: causation_id - an optional UUID used to identify the cause of the command being dispatched. command_uuid - an optional UUID used to identify the command being dispatched. correlation_id - an optional UUID used to correlate related commands/events together. consistency - one of :eventual (default) or :strong . By setting the consistency to :strong a successful command dispatch will block until all strongly consistent event handlers and process managers have handled all events created by the command. metadata - an optional map containing key/value pairs comprising the metadata to be associated with all events created by the command. returning - to choose what response is returned from a successful command dispatch. The default is to return an :ok . The available options are: :aggregate_state - to return the update aggregate state in the successful response: {:ok, aggregate_state} . :aggregate_version - to include the aggregate stream version in the successful response: {:ok, aggregate_version} . :events - to return the resultant domain events. An empty list will be returned if no events were produced. :execution_result - to return a Commanded.Commands.ExecutionResult struct containing the aggregate's identity, version, and any events produced from the command along with their associated metadata. false - don't return anything except an :ok . timeout - as described above. Returns :ok on success unless the :returning option is specified where it returns one of {:ok, aggregate_state} , {:ok, aggregate_version} , or {:ok, %Commanded.Commands.ExecutionResult{}} . Returns {:error, reason} on failure. Example command = % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } :ok = BankRouter . dispatch ( command , consistency : :strong , timeout : 30_000 )","ref":"Commanded.Commands.Router.html#c:dispatch/2","title":"Commanded.Commands.Router.dispatch/2","type":"callback"},{"doc":"Define an aggregate's identity You can define the identity field for an aggregate using the identify macro. The configured identity will be used for all commands registered to the aggregate, unless overridden by a command registration. Example defmodule BankRouter do use Commanded.Commands.Router identify BankAccount , by : :account_number , prefix : &quot;bank-account-&quot; end","ref":"Commanded.Commands.Router.html#identify/2","title":"Commanded.Commands.Router.identify/2","type":"macro"},{"doc":"Include the given middleware module to be called before and after success or failure of each command dispatch The middleware module must implement the Commanded.Middleware behaviour. Middleware modules are executed in the order they are defined. Example defmodule BankRouter do use Commanded.Commands.Router middleware CommandLogger middleware MyCommandValidator middleware AuthorizeCommand dispatch [ OpenAccount , DepositMoney ] , to : BankAccount , identity : :account_number end","ref":"Commanded.Commands.Router.html#middleware/1","title":"Commanded.Commands.Router.middleware/1","type":"macro"},{"doc":"Data related to an event handling failure. The available fields are: :application - the associated Commanded.Application . :handler_name - the name of the event handler. :handler_state - optional event handler state. :context - a map that is passed between each failure. Use it to store any transient state between failures. As an example it could be used to count error failures and stop or skip the problematic event after too many. :metadata - the metadata associated with the failed event. :stacktrace - the stacktrace if the error was an unhandled exception.","ref":"Commanded.Event.FailureContext.html","title":"Commanded.Event.FailureContext","type":"module"},{"doc":"","ref":"Commanded.Event.FailureContext.html#t:t/0","title":"Commanded.Event.FailureContext.t/0","type":"type"},{"doc":"Defines the behaviour an event handler must implement and provides a convenience macro that implements the behaviour, allowing you to handle only the events you are interested in processing. You should start your event handlers using a Supervisor to ensure they are restarted on error. Example defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : &quot;ExampleHandler&quot; def handle ( % AnEvent { .. } , _metadata ) do # ... process the event :ok end end Start your event handler process (or use a Supervisor ): { :ok , _handler } = ExampleHandler . start_link ( ) Event handler name The name you specify is used when subscribing to the event store. You must use a unique name for each event handler and process manager you start. Also, you should not change the name once the handler has been deployed. A new subscription will be created if you change the name and the event handler will receive already handled events. You can use the module name of your event handler using the __MODULE__ special form: defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : __MODULE__ end Subscription options You can choose to start the event handler's event store subscription from :origin , :current position, or an exact event number using the start_from option. The default is to use the origin so your handler will receive all events. Use the :current position when you don't want newly created event handlers to go through all previous events. An example would be adding an event handler to send transactional emails to an already deployed system containing many historical events. The start_from option only applies when the subscription is initially created, the first time the handler starts. Whenever the handler restarts the subscription will resume from the next event after the last successfully processed event. Restarting an event handler does not restart its subscription. Example Set the start_from option ( :origin , :current , or an explicit event number) when using Commanded.Event.Handler : defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : &quot;ExampleHandler&quot; , start_from : :origin end You can optionally override :start_from by passing it as option when starting your handler: { :ok , _handler } = ExampleHandler . start_link ( start_from : :current ) Subscribing to an individual stream By default event handlers will subscribe to all events appended to any stream. Provide a subscribe_to option to subscribe to a single stream. defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : __MODULE__ , subscribe_to : &quot;stream1234&quot; end This will ensure the handler only receives events appended to that stream. Runtime event handler configuration Runtime options can be provided to the event handler's start_link/1 function or its child spec. The init/1 callback function can also be used to define runtime configuration. Example Provide runtime configuration to start_link/1 : { :ok , _pid } = ExampleHandler . start_link ( application : ExampleApp , name : &quot;ExampleHandler&quot; ) Or when supervised: Supervisor . start_link ( [ { ExampleHandler , application : ExampleApp , name : &quot;ExampleHandler&quot; } ] , strategy : :one_for_one ) Event handler state An event handler can define and update state which is held in the GenServer process memory. It is passed to the handle/2 function as part of the metadata using the :state key. The state is transient and will be lost whenever the process restarts. Initial state can be set in the init/1 callback function by adding a :state key to the config. It can also be provided when starting the handler process: ExampleHandler . start_link ( state : initial_state ) Or when supervised: Supervisor . start_link ( [ { ExampleHandler , state : initial_state } ] , strategy : :one_for_one ) State can be updated by returning {:ok, new_state} from any handle/2 function. Returning an :ok reply will keep the state unchanged. Handler state is also included in the Commanded.Event.FailureContext struct passed to the error/3 callback function. Example defmodule StatefulEventHandler do use Commanded.Event.Handler , application : ExampleApp , name : __MODULE__ def init ( config ) do config = Keyword . put_new ( config , :state , %{ } ) { :ok , config } end def handle ( event , metadata ) do %{ state : state } = metadata new_state = mutate_state ( state ) { :ok , new_state } end end Concurrency An event handler may be configured to start multiple processes to handle the events concurrently. By default one process will be started, processing events one at a time in order. The :concurrency option determines how many event handler processes are started. It must be a positive integer. Note with concurrent processing events will likely by processed out of order. If you need to enforce an order, such as per stream or by using a field from an event, you can define a c:partition_by/1 callback function in the event handler module. The function will receive each event and its metadata and must return a consistent term indicating the event's partition. Events which return the same term are guaranteed to be processed in order by the same event handler instance. While events with different partitions may be processed concurrently by another instance. An attempt will be made to distribute events as evenly as possible to all running event handler instances. Only :eventual consistency is supported when multiple handler processes are configured with a :concurrency of greater than one. Example defmodule ConcurrentProcssingEventHandler do alias Commanded.EventStore.RecordedEvent use Commanded.Event.Handler , application : ExampleApp , name : __MODULE__ , concurrency : 10 def init ( config ) do # Fetch the index of this event handler instance (0..9 in this example) index = Keyword . fetch! ( config , :index ) { :ok , config } end def handle ( event , metadata ) do :ok end # Partition events by their stream def partition_by ( event , metadata ) do %{ stream_id : stream_id } = metadata stream_id end end Consistency For each event handler you can define its consistency, as one of either :strong or :eventual . This setting is used when dispatching commands and specifying the consistency option. When you dispatch a command using :strong consistency, after successful command dispatch the process will block until all event handlers configured to use :strong consistency have processed the domain events created by the command. This is useful when you have a read model updated by an event handler that you wish to query for data affected by the command dispatch. With :strong consistency you are guaranteed that the read model will be up-to-date after the command has successfully dispatched. It can be safely queried for data updated by any of the events created by the command. The default setting is :eventual consistency. Command dispatch will return immediately upon confirmation of event persistence, not waiting for any event handlers. Note strong consistency does not imply a transaction covers the command dispatch and event handling. It only guarantees that the event handler will have processed all events produced by the command: if event handling fails the events will have still been persisted. Example Define an event handler with :strong consistency: defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : &quot;ExampleHandler&quot; , consistency : :strong end Dynamic application An event handler's application can be provided as an option to start_link/1 . This can be used to start the same handler multiple times, each using a separate Commanded application and event store. Example Start an event handler process for each tenant in a multi-tenanted app, guaranteeing that the data and processing remains isolated between tenants. for tenant &lt;- [ :tenant1 , :tenant2 , :tenant3 ] do { :ok , _app } = MyApp.Application . start_link ( name : tenant ) { :ok , _handler } = ExampleHandler . start_link ( application : tenant ) end Typically you would start the event handlers using a supervisor: children = for tenant &lt;- [ :tenant1 , :tenant2 , :tenant3 ] do { ExampleHandler , application : tenant } end Supervisor . start_link ( children , strategy : :one_for_one ) The above example requires three named Commanded applications to have already been started. Telemetry [:commanded, :event, :handle, :start] Description: Emitted when an event handler starts handling an event Measurements: %{system_time: integer()} Metadata: %{application: Commanded.Application.t(), context: map(), handler_name: String.t(), handler_module: atom(), handler_state: map(), recorded_event: RecordedEvent.t()} [:commanded, :event, :handle, :stop] Description: Emitted when an event handler stops handling an event Measurements: %{duration: non_neg_integer()} Metadata: %{:application =&gt; Commanded.Application.t(), :context =&gt; map(), :handler_name =&gt; String.t(), :handler_module =&gt; atom(), :handler_state =&gt; map(), :recorded_event =&gt; RecordedEvent.t(), optional(:error) =&gt; any()} [:commanded, :event, :handle, :exception] Description: Emitted when an event handler raises an exception Measurements: %{duration: non_neg_integer()} Metadata: %{application: Commanded.Application.t(), context: map(), handler_name: String.t(), handler_module: atom(), handler_state: map(), recorded_event: RecordedEvent.t(), kind: :throw | :error | :exit, reason: any(), stacktrace: list()}","ref":"Commanded.Event.Handler.html","title":"Commanded.Event.Handler","type":"behaviour"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Commanded.Event.Handler.html#child_spec/1","title":"Commanded.Event.Handler.child_spec/1","type":"function"},{"doc":"Called when an event handle/2 callback returns an error. The error/3 function allows you to control how event handling failures are handled. The function is passed the error returned by the event handler (e.g. {:error, :failure} ), the event causing the error, and a context map containing state passed between retries. Use pattern matching on the error and/or failed event to explicitly handle certain errors or events. Use the context map to track any transient state you need to access between retried failures. You can return one of the following responses depending upon the error severity: {:retry, context} - retry the failed event, provide a context map, or updated Commanded.Event.FailureContext struct, containing any state to be passed to subsequent failures. This could be used to count the number of failures, stopping after too many. {:retry, delay, context} - retry the failed event, after sleeping for the requested delay (in milliseconds). Context is a map or Commanded.Event.FailureContext struct as described in {:retry, context} above. :skip - skip the failed event by acknowledging receipt. {:stop, reason} - stop the event handler with the given reason. The default behaviour if you don't provide an error/3 callback is to stop the event handler using the exact error reason returned from the handle/2 function. If the event handler is supervised using restart permanent or transient stopping on error will cause the handler to be restarted. It will likely crash again as it will reprocesse the problematic event. This can lead to cascading failures going up the supervision tree. Example error handling defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : __MODULE__ require Logger alias Commanded.Event.FailureContext def handle ( % AnEvent { } , _metadata ) do # simulate event handling failure { :error , :failed } end def error ( { :error , :failed } , % AnEvent { } = event , % FailureContext { context : context } ) do context = record_failure ( context ) case Map . get ( context , :failures ) do too_many when too_many &gt;= 3 -&gt; # skip bad event after third failure Logger . warn ( fn -&gt; &quot;Skipping bad event, too many failures: &quot; &lt;&gt; inspect ( event ) end ) :skip _ -&gt; # retry event, failure count is included in context map { :retry , context } end end defp record_failure ( context ) do Map . update ( context , :failures , 1 , fn failures -&gt; failures + 1 end ) end end","ref":"Commanded.Event.Handler.html#c:error/3","title":"Commanded.Event.Handler.error/3","type":"callback"},{"doc":"Handle a domain event and its metadata. Return :ok on success, {:error, :already_seen_event} to ack and skip the event, or {:error, reason} on failure.","ref":"Commanded.Event.Handler.html#c:handle/2","title":"Commanded.Event.Handler.handle/2","type":"callback"},{"doc":"Optional initialisation callback function called when the handler starts. Can be used to start any related processes when the event handler is started. This callback function must return :ok , or {:stop, reason} to stop the handler process. Any other return value will terminate the event handler with an error. Example defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : &quot;ExampleHandler&quot; # Optional initialisation def init do :ok end def handle ( % AnEvent { .. } , _metadata ) do # Process the event ... :ok end end","ref":"Commanded.Event.Handler.html#c:init/0","title":"Commanded.Event.Handler.init/0","type":"callback"},{"doc":"Optional callback function called to configure the handler before it starts. It is passed the merged compile-time and runtime config, and must return the updated config as {:ok, config} . Note this function is called before the event handler process is started and is not run from the handler's process. You cannot use self() to access the handler's PID. Example The init/1 function is used to define the handler's application and name based upon a value provided at runtime: defmodule ExampleHandler do use Commanded.Event.Handler def init ( config ) do { tenant , config } = Keyword . pop! ( config , :tenant ) config = config |&gt; Keyword . put ( :application , Module . concat ( [ ExampleApp , tenant ] ) ) |&gt; Keyword . put ( :name , Module . concat ( [ __MODULE__ , tenant ] ) ) { :ok , config } end end Usage: { :ok , _pid } = ExampleHandler . start_link ( tenant : :tenant1 )","ref":"Commanded.Event.Handler.html#c:init/1","title":"Commanded.Event.Handler.init/1","type":"callback"},{"doc":"Determine which partition an event belongs to. Only applicable when an event handler has been configured with more than one instance via the :concurrency option.","ref":"Commanded.Event.Handler.html#c:partition_by/2","title":"Commanded.Event.Handler.partition_by/2","type":"callback"},{"doc":"","ref":"Commanded.Event.Handler.html#t:consistency/0","title":"Commanded.Event.Handler.consistency/0","type":"type"},{"doc":"","ref":"Commanded.Event.Handler.html#t:domain_event/0","title":"Commanded.Event.Handler.domain_event/0","type":"type"},{"doc":"","ref":"Commanded.Event.Handler.html#t:metadata/0","title":"Commanded.Event.Handler.metadata/0","type":"type"},{"doc":"","ref":"Commanded.Event.Handler.html#t:subscribe_from/0","title":"Commanded.Event.Handler.subscribe_from/0","type":"type"},{"doc":"Map events to/from the structs used for persistence. Example Map domain event structs to Commanded.EventStore.EventData structs in preparation for appending to the configured event store: events = [ % ExampleEvent1 { } , % ExampleEvent2 { } ] event_data = Commanded.Event.Mapper . map_to_event_data ( events ) :ok = Commanded.EventStore . append_to_stream ( &quot;stream-1234&quot; , :any_version , event_data )","ref":"Commanded.Event.Mapper.html","title":"Commanded.Event.Mapper","type":"module"},{"doc":"Map an Commanded.EventStore.RecordedEvent struct to its event data.","ref":"Commanded.Event.Mapper.html#map_from_recorded_event/1","title":"Commanded.Event.Mapper.map_from_recorded_event/1","type":"function"},{"doc":"Map a list of Commanded.EventStore.RecordedEvent structs to their event data.","ref":"Commanded.Event.Mapper.html#map_from_recorded_events/1","title":"Commanded.Event.Mapper.map_from_recorded_events/1","type":"function"},{"doc":"Map a domain event (or list of events) to an Commanded.EventStore.EventData struct (or list of structs). Optionally, include the causation_id , correlation_id , and metadata associated with the event(s). Examples event_data = Commanded.Event.Mapper . map_to_event_data ( % ExampleEvent { } ) event_data = Commanded.Event.Mapper . map_to_event_data ( [ % ExampleEvent1 { } , % ExampleEvent2 { } ] , causation_id : UUID . uuid4 ( ) , correlation_id : UUID . uuid4 ( ) , metadata : %{ &quot;user_id&quot; =&gt; user_id } )","ref":"Commanded.Event.Mapper.html#map_to_event_data/2","title":"Commanded.Event.Mapper.map_to_event_data/2","type":"function"},{"doc":"","ref":"Commanded.Event.Mapper.html#t:event/0","title":"Commanded.Event.Mapper.event/0","type":"type"},{"doc":"Protocol to allow an event to be transformed before being passed to a consumer. You can use an upcaster to change the shape of an event (e.g. add a new field with a default, rename a field) or rename an event. Because the upcaster changes any historical event to the latest version, consumers (aggregates, event handlers, and process managers) only need to support the latest version. Example defimpl Commanded.Event.Upcaster , for : AnEvent do def upcast ( % AnEvent { } = event , _metadata ) do % AnEvent { name : name } = event % AnEvent { event | first_name : name } end end Metadata The upcast/2 function receives the domain event and a map of metadata associated with that event. The metadata is provided during command dispatch. In addition to the metadata key/values you provide, the following system values will be included in the metadata: application - the Commanded.Application used to read the event. event_id - a globally unique UUID to identify the event. event_number - a globally unique, monotonically incrementing integer used to order the event amongst all events. stream_id - the stream identity for the event. stream_version - the version of the stream for the event. causation_id - an optional UUID identifier used to identify which command caused the event. correlation_id - an optional UUID identifier used to correlate related commands/events. created_at - the datetime, in UTC, indicating when the event was created. These key/value metadata pairs will use atom keys to differentiate them from the user provided metadata which uses string keys.","ref":"Commanded.Event.Upcaster.html","title":"Commanded.Event.Upcaster","type":"protocol"},{"doc":"","ref":"Commanded.Event.Upcaster.html#upcast/2","title":"Commanded.Event.Upcaster.upcast/2","type":"function"},{"doc":"","ref":"Commanded.Event.Upcaster.html#t:t/0","title":"Commanded.Event.Upcaster.t/0","type":"type"},{"doc":"Data related to a process manager event handling or command dispatch failure. The available fields are: context - the context map passed between each failure and may be used to track state between retries, such as to count failures. last_event - the last event the process manager received. pending_commands - the pending commands that were not executed yet. process_manager_state - the state the process manager would be in if the command would not fail. stacktrace - the stacktrace if the error was an unhandled exception.","ref":"Commanded.ProcessManagers.FailureContext.html","title":"Commanded.ProcessManagers.FailureContext","type":"module"},{"doc":"","ref":"Commanded.ProcessManagers.FailureContext.html#t:t/0","title":"Commanded.ProcessManagers.FailureContext.t/0","type":"type"},{"doc":"Macro used to define a process manager. A process manager is responsible for coordinating one or more aggregates. It handles events and dispatches commands in response. Process managers have state that can be used to track which aggregates are being orchestrated. Process managers can be used to implement long-running transactions by following the saga pattern. This is a sequence of commands and their compensating commands which can be used to rollback on failure. Use the Commanded.ProcessManagers.ProcessManager macro in your process manager module and implement the callback functions defined in the behaviour: interested?/1 handle/2 apply/2 c:after_command/2 - c:error/3 Please read the [Process managers](process-managers.html) guide for more detail. ### Example defmodule ExampleProcessManager do use Commanded.ProcessManagers.ProcessManager, application: ExampleApp, name: &quot;ExampleProcessManager&quot; defstruct [] def interested?(%AnEvent{uuid: uuid}), do: {:start, uuid} def handle(%ExampleProcessManager{}, %ExampleEvent{}) do [ %ExampleCommand{} ] end def after_command(%ExampleProcessManager{}, %ExampleCommand{}) do :stop end def error({:error, failure}, %ExampleEvent{}, _failure_context) do # Retry, skip, ignore, or stop process manager on error handling event end def error({:error, failure}, %ExampleCommand{}, _failure_context) do # Retry, skip, ignore, or stop process manager on error dispatching command end end Start the process manager (or configure as a worker inside a [Supervisor](supervision.html)) {:ok, process_manager} = ExampleProcessManager.start_link() ## c:init/1 callback An c:init/1 function can be defined in your process manager which is used to provide runtime configuration. This callback function must return with the updated config. ### Example The c:init/1 function is used to define the process manager's application and name based upon a value provided at runtime: defmodule ExampleProcessManager do use Commanded.ProcessManagers.ProcessManager def init(config) do {tenant, config} = Keyword.pop!(config, :tenant) config = config |&gt; Keyword.put(:application, Module.concat([ExampleApp, tenant])) |&gt; Keyword.put(:name, Module.concat([__MODULE__, tenant])) {:ok, config} end end Usage: {:ok, _pid} = ExampleProcessManager.start_link(tenant: :tenant1) ## Error handling You can define an c:error/3 callback function to handle any errors or exceptions during event handling or returned by commands dispatched from your process manager. The function is passed the error (e.g. ), the failed event or command, and a failure context. See Commanded.ProcessManagers.FailureContext for details. Use pattern matching on the error and/or failed event/command to explicitly handle certain errors, events, or commands. You can choose to retry, skip, ignore, or stop the process manager after a command dispatch error. The default behaviour, if you don't provide an c:error/3 callback, is to stop the process manager using the exact error reason returned from the event handler function or command dispatch. You should supervise your process managers to ensure they are restarted on error. ### Example defmodule ExampleProcessManager do use Commanded.ProcessManagers.ProcessManager, application: ExampleApp, name: &quot;ExampleProcessManager&quot; # stop process manager after three failures def error({:error, _failure}, _failed_command, %{context: %{failures: failures}}) when failures &gt;= 2 do {:stop, :too_many_failures} end # retry command, record failure count in context map def error({:error, _failure}, _failed_command, %{context: context}) do context = Map.update(context, :failures, 1, fn failures -&gt; failures + 1 end) {:retry, context} end end ## Idle process timeouts Each instance of a process manager will run indefinitely once started. To reduce memory usage you can configure an idle timeout, in milliseconds, after which the process will be shutdown. The process will be restarted whenever another event is routed to it and its state will be rehydrated from the instance snapshot. ### Example defmodule ExampleProcessManager do use Commanded.ProcessManagers.ProcessManager, application: ExampleApp, name: &quot;ExampleProcessManager&quot; idle_timeout: :timer.minutes(10) end ## Event handling timeout You can configure a timeout for event handling to ensure that events are processed in a timely manner without getting stuck. An event_timeout option, defined in milliseconds, may be provided when using the Commanded.ProcessManagers.ProcessManager macro at compile time: defmodule TransferMoneyProcessManager do use Commanded.ProcessManagers.ProcessManager, application: ExampleApp, name: &quot;TransferMoneyProcessManager&quot;, router: BankRouter, event_timeout: :timer.minutes(10) end Or may be configured when starting a process manager: {:ok, _pid} = TransferMoneyProcessManager.start_link( event_timeout: :timer.hours(1) ) After the timeout has elapsed, indicating the process manager has not processed an event within the configured period, the process manager is stopped. The process manager will be restarted if supervised and will retry the event, this should help resolve transient problems. ## Consistency For each process manager you can define its consistency, as one of either :strong or :eventual . This setting is used when dispatching commands and specifying the consistency option. When you dispatch a command using :strong consistency, after successful command dispatch the process will block until all process managers configured to use :strong consistency have processed the domain events created by the command. The default setting is :eventual consistency. Command dispatch will return immediately upon confirmation of event persistence, not waiting for any process managers. ### Example Define a process manager with :strong consistency: defmodule ExampleProcessManager do use Commanded.ProcessManagers.ProcessManager, application: ExampleApp, name: &quot;ExampleProcessManager&quot;, consistency: :strong end ## Dynamic application A process manager's application can be provided as an option to start_link/1 . This can be used to start the same process manager multiple times, each using a separate Commanded application and event store. ### Example Start an process manager for each tenant in a multi-tenanted app, guaranteeing that the data and processing remains isolated between tenants. for tenant &lt;- [:tenant1, :tenant2, :tenant3] do {:ok, _app} = MyApp.Application.start_link(name: tenant) {:ok, _handler} = ExampleProcessManager.start_link(application: tenant) end Typically you would start the event handlers using a supervisor: children = for tenant &lt;- [:tenant1, :tenant2, :tenant3] do {ExampleProcessManager, application: tenant} end Supervisor.start_link(children, strategy: :one_for_one) The above example requires three named Commanded applications to have already been started. ## Telemetry * [:commanded, :process_manager, :handle, :start] * Description: Emitted when a process manager starts handling an event * Measurements: %{system_time: integer()} * Metadata: %{application: Commanded.Application.t(), process_manager_name: String.t() | Inspect.t(), process_manager_module: module(), process_state: term(), process_uuid: String.t()} * [:commanded, :process_manager, :handle, :stop] * Description: Emitted when a process manager stops handling an event * Measurements: %{system_time: integer()} * Metadata: %{application: Commanded.Application.t(), commands: [struct()], error: nil | any(), process_manager_name: String.t() | Inspect.t(), process_manager_module: module(), process_state: term(), process_uuid: String.t()} * [:commanded, :process_manager, :handle, :exception] * Description: Emitted when a process manager raises an exception * Measurements: %{system_time: integer()} * Metadata: %{application: Commanded.Application.t(), process_manager_name: String.t() | Inspect.t(), process_manager_module: module(), process_state: term(), process_uuid: String.t(), kind: :throw | :error | :exit, reason: any(), stacktrace: list()} `","ref":"Commanded.ProcessManagers.ProcessManager.html","title":"Commanded.ProcessManagers.ProcessManager","type":"behaviour"},{"doc":"Stop the process manager instance after a command is successfully dispatched. The after_command/2 function can be omitted if you do not need to stop after a specific command or if you would instead use the interested?/1 stop mechanism.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:after_command/2","title":"Commanded.ProcessManagers.ProcessManager.after_command/2","type":"callback"},{"doc":"Mutate the process manager's state by applying the domain event. The apply/2 function is used to mutate the process manager's state. It receives the current state and the domain event, and must return the modified state. This callback function is optional, the default behaviour is to retain the process manager's current state.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:apply/2","title":"Commanded.ProcessManagers.ProcessManager.apply/2","type":"callback"},{"doc":"Called when a command dispatch or event handling returns an error. The error/3 function allows you to control how event handling and command dispatch and failures are handled. The function is passed the error (e.g. {:error, :failure} ), the failed event (during failed event handling) or failed command (during failed dispatch), and a failure context struct (see Commanded.ProcessManagers.FailureContext for details). The failure context contains a context map you can use to pass transient state between failures. For example it can be used to count the number of failures. You can return one of the following responses depending upon the error severity: {:retry, context} - retry the failed command, provide a context map or Commanded.ProcessManagers.FailureContext struct, containing any state passed to subsequent failures. This could be used to count the number of retries, failing after too many attempts. {:retry, delay, context} - retry the failed command, after sleeping for the requested delay (in milliseconds). Context is a map or Commanded.ProcessManagers.FailureContext as described in {:retry, context} above. {:stop, reason} - stop the process manager with the given reason. For event handling failures, when failure source is an event, you can also return: :skip - to skip the problematic event. No commands will be dispatched. For command dispatch failures, when failure source is a command, you can also return: :skip - skip the failed command and continue dispatching any pending commands. {:skip, :continue_pending} - skip the failed command, but continue dispatching any pending commands. {:skip, :discard_pending} - discard the failed command and any pending commands. {:continue, commands, context} - continue dispatching the given commands. This allows you to retry the failed command, modify it and retry, drop it or drop all pending commands by passing an empty list [] . Context is a map as described in {:retry, context} above.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:error/3","title":"Commanded.ProcessManagers.ProcessManager.error/3","type":"callback"},{"doc":"Process manager instance handles a domain event, returning any commands to dispatch. A handle/2 function can be defined for each :start and :continue tagged event previously specified. It receives the process manager's state and the event to be handled. It must return the commands to be dispatched. This may be none, a single command, or many commands. The handle/2 function can be omitted if you do not need to dispatch a command and are only mutating the process manager's state.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:handle/2","title":"Commanded.ProcessManagers.ProcessManager.handle/2","type":"callback"},{"doc":"Get the identity of the current process instance. This must only be called within a process manager's handle/2 or apply/2 callback function. Example defmodule ExampleProcessManager do use Commanded.ProcessManagers.ProcessManager , application : MyApp.Application , name : __MODULE__ def interested? ( % ProcessStarted { uuids : uuids } ) , do : { :start , uuids } def handle ( % IdentityProcessManager { } , % ProcessStarted { } = event ) do # Identify which uuid is associated with the current instance from the # list of uuids in the event. uuid = Commanded.ProcessManagers.ProcessManager . identity ( ) # ... end end","ref":"Commanded.ProcessManagers.ProcessManager.html#identity/0","title":"Commanded.ProcessManagers.ProcessManager.identity/0","type":"function"},{"doc":"Optional callback function called to configure the process manager before it starts. It is passed the merged compile-time and runtime config, and must return the updated config.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:init/1","title":"Commanded.ProcessManagers.ProcessManager.init/1","type":"callback"},{"doc":"Is the process manager interested in the given command? The interested?/1 function is used to indicate which events the process manager receives. The response is used to route the event to an existing instance or start a new process instance: {:start, process_uuid} - create a new instance of the process manager. {:start!, process_uuid} - create a new instance of the process manager (strict). {:continue, process_uuid} - continue execution of an existing process manager. {:continue!, process_uuid} - continue execution of an existing process manager (strict). {:stop, process_uuid} - stop an existing process manager, shutdown its process, and delete its persisted state. false - ignore the event. You can return a list of process identifiers when a single domain event is to be handled by multiple process instances. Strict process routing Using strict routing, with :start! or :continue , enforces the following validation checks: {:start!, process_uuid} - validate process does not already exist. {:continue!, process_uuid} - validate process already exists. If the check fails an error will be passed to the error/3 callback function: {:error, {:start!, :process_already_started}} {:error, {:continue!, :process_not_started}} The error/3 function can choose to :stop the process or :skip the problematic event.","ref":"Commanded.ProcessManagers.ProcessManager.html#c:interested?/1","title":"Commanded.ProcessManagers.ProcessManager.interested?/1","type":"callback"},{"doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#parse_config!/2","title":"Commanded.ProcessManagers.ProcessManager.parse_config!/2","type":"function"},{"doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:command/0","title":"Commanded.ProcessManagers.ProcessManager.command/0","type":"type"},{"doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:consistency/0","title":"Commanded.ProcessManagers.ProcessManager.consistency/0","type":"type"},{"doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:domain_event/0","title":"Commanded.ProcessManagers.ProcessManager.domain_event/0","type":"type"},{"doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:process_manager/0","title":"Commanded.ProcessManagers.ProcessManager.process_manager/0","type":"type"},{"doc":"","ref":"Commanded.ProcessManagers.ProcessManager.html#t:process_uuid/0","title":"Commanded.ProcessManagers.ProcessManager.process_uuid/0","type":"type"},{"doc":"Use the event store configured for a Commanded application. Telemetry Events Adds telemetry events for the following functions. Events are emitted in the form [:commanded, :event_store, event] with their spannable postfixes ( start , stop , exception ) ack_event/3 adapter/2 append_to_stream/4 delete_snapshot/2 delete_subscription/3 read_snapshot/2 record_snapshot/2 stream_forward/2 stream_forward/3 stream_forward/4 subscribe/2 subscribe_to/5 subscribe_to/6 unsubscribe/2","ref":"Commanded.EventStore.html","title":"Commanded.EventStore","type":"module"},{"doc":"Acknowledge receipt and successful processing of the given event received from a subscription to an event stream.","ref":"Commanded.EventStore.html#ack_event/3","title":"Commanded.EventStore.ack_event/3","type":"function"},{"doc":"Get the configured event store adapter for the given application.","ref":"Commanded.EventStore.html#adapter/2","title":"Commanded.EventStore.adapter/2","type":"function"},{"doc":"Append one or more events to a stream atomically.","ref":"Commanded.EventStore.html#append_to_stream/4","title":"Commanded.EventStore.append_to_stream/4","type":"function"},{"doc":"Delete a previously recorded snapshot for a given source","ref":"Commanded.EventStore.html#delete_snapshot/2","title":"Commanded.EventStore.delete_snapshot/2","type":"function"},{"doc":"Delete an existing subscription. Example :ok = Commanded.EventStore . delete_subscription ( MyApp , :all , &quot;Example&quot; )","ref":"Commanded.EventStore.html#delete_subscription/3","title":"Commanded.EventStore.delete_subscription/3","type":"function"},{"doc":"Read a snapshot, if available, for a given source.","ref":"Commanded.EventStore.html#read_snapshot/2","title":"Commanded.EventStore.read_snapshot/2","type":"function"},{"doc":"Record a snapshot of the data and metadata for a given source","ref":"Commanded.EventStore.html#record_snapshot/2","title":"Commanded.EventStore.record_snapshot/2","type":"function"},{"doc":"Streams events from the given stream, in the order in which they were originally written.","ref":"Commanded.EventStore.html#stream_forward/4","title":"Commanded.EventStore.stream_forward/4","type":"function"},{"doc":"Create a transient subscription to a single event stream. The event store will publish any events appended to the given stream to the subscriber process as an {:events, events} message. The subscriber does not need to acknowledge receipt of the events.","ref":"Commanded.EventStore.html#subscribe/2","title":"Commanded.EventStore.subscribe/2","type":"function"},{"doc":"Create a persistent subscription to an event stream. To subscribe to all events appended to any stream use :all as the stream when subscribing. The event store will remember the subscribers last acknowledged event. Restarting the named subscription will resume from the next event following the last seen. Once subscribed, the subscriber process should be sent a {:subscribed, subscription} message to allow it to defer initialisation until the subscription has started. The subscriber process will be sent all events persisted to the stream. It will receive a {:events, events} message for each batch of events persisted for a single aggregate. The subscriber must ack each received, and successfully processed event, using Commanded.EventStore.ack_event/3 . Examples Subscribe to all streams: { :ok , subscription } = Commanded.EventStore . subscribe_to ( MyApp , :all , &quot;Example&quot; , self ( ) , :current ) Subscribe to a single stream: { :ok , subscription } = Commanded.EventStore . subscribe_to ( MyApp , &quot;stream1&quot; , &quot;Example&quot; , self ( ) , :origin )","ref":"Commanded.EventStore.html#subscribe_to/6","title":"Commanded.EventStore.subscribe_to/6","type":"function"},{"doc":"Unsubscribe an existing subscriber from event notifications. This will not delete the subscription. Example :ok = Commanded.EventStore . unsubscribe ( MyApp , subscription )","ref":"Commanded.EventStore.html#unsubscribe/2","title":"Commanded.EventStore.unsubscribe/2","type":"function"},{"doc":"","ref":"Commanded.EventStore.html#t:application/0","title":"Commanded.EventStore.application/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.html#t:config/0","title":"Commanded.EventStore.config/0","type":"type"},{"doc":"Defines the behaviour to be implemented by an event store adapter to be used by Commanded.","ref":"Commanded.EventStore.Adapter.html","title":"Commanded.EventStore.Adapter","type":"behaviour"},{"doc":"Acknowledge receipt and successful processing of the given event received from a subscription to an event stream.","ref":"Commanded.EventStore.Adapter.html#c:ack_event/3","title":"Commanded.EventStore.Adapter.ack_event/3","type":"callback"},{"doc":"Append one or more events to a stream atomically.","ref":"Commanded.EventStore.Adapter.html#c:append_to_stream/4","title":"Commanded.EventStore.Adapter.append_to_stream/4","type":"callback"},{"doc":"Return a child spec defining all processes required by the event store.","ref":"Commanded.EventStore.Adapter.html#c:child_spec/2","title":"Commanded.EventStore.Adapter.child_spec/2","type":"callback"},{"doc":"Delete a previously recorded snapshot for a given source","ref":"Commanded.EventStore.Adapter.html#c:delete_snapshot/2","title":"Commanded.EventStore.Adapter.delete_snapshot/2","type":"callback"},{"doc":"Delete an existing subscription.","ref":"Commanded.EventStore.Adapter.html#c:delete_subscription/3","title":"Commanded.EventStore.Adapter.delete_subscription/3","type":"callback"},{"doc":"Read a snapshot, if available, for a given source.","ref":"Commanded.EventStore.Adapter.html#c:read_snapshot/2","title":"Commanded.EventStore.Adapter.read_snapshot/2","type":"callback"},{"doc":"Record a snapshot of the data and metadata for a given source","ref":"Commanded.EventStore.Adapter.html#c:record_snapshot/2","title":"Commanded.EventStore.Adapter.record_snapshot/2","type":"callback"},{"doc":"Streams events from the given stream, in the order in which they were originally written.","ref":"Commanded.EventStore.Adapter.html#c:stream_forward/4","title":"Commanded.EventStore.Adapter.stream_forward/4","type":"callback"},{"doc":"Create a transient subscription to a single event stream. The event store will publish any events appended to the given stream to the subscriber process as an {:events, events} message. The subscriber does not need to acknowledge receipt of the events.","ref":"Commanded.EventStore.Adapter.html#c:subscribe/2","title":"Commanded.EventStore.Adapter.subscribe/2","type":"callback"},{"doc":"Create a persistent subscription to an event stream.","ref":"Commanded.EventStore.Adapter.html#c:subscribe_to/6","title":"Commanded.EventStore.Adapter.subscribe_to/6","type":"callback"},{"doc":"Unsubscribe an existing subscriber from event notifications. This should not delete the subscription.","ref":"Commanded.EventStore.Adapter.html#c:unsubscribe/2","title":"Commanded.EventStore.Adapter.unsubscribe/2","type":"callback"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:adapter_meta/0","title":"Commanded.EventStore.Adapter.adapter_meta/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:application/0","title":"Commanded.EventStore.Adapter.application/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:config/0","title":"Commanded.EventStore.Adapter.config/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:error/0","title":"Commanded.EventStore.Adapter.error/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:expected_version/0","title":"Commanded.EventStore.Adapter.expected_version/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:options/0","title":"Commanded.EventStore.Adapter.options/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:source_uuid/0","title":"Commanded.EventStore.Adapter.source_uuid/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:start_from/0","title":"Commanded.EventStore.Adapter.start_from/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:stream_uuid/0","title":"Commanded.EventStore.Adapter.stream_uuid/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:subscriber/0","title":"Commanded.EventStore.Adapter.subscriber/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:subscription/0","title":"Commanded.EventStore.Adapter.subscription/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.Adapter.html#t:subscription_name/0","title":"Commanded.EventStore.Adapter.subscription_name/0","type":"type"},{"doc":"An in-memory event store adapter implemented as a GenServer process which stores events in memory only. This is only designed for testing purposes.","ref":"Commanded.EventStore.Adapters.InMemory.html","title":"Commanded.EventStore.Adapters.InMemory","type":"module"},{"doc":"Returns a specification to start this module under a supervisor. See Supervisor .","ref":"Commanded.EventStore.Adapters.InMemory.html#child_spec/1","title":"Commanded.EventStore.Adapters.InMemory.child_spec/1","type":"function"},{"doc":"","ref":"Commanded.EventStore.Adapters.InMemory.html#reset!/2","title":"Commanded.EventStore.Adapters.InMemory.reset!/2","type":"function"},{"doc":"","ref":"Commanded.EventStore.Adapters.InMemory.html#start_link/1","title":"Commanded.EventStore.Adapters.InMemory.start_link/1","type":"function"},{"doc":"EventData contains the data for a single event before being persisted to storage.","ref":"Commanded.EventStore.EventData.html","title":"Commanded.EventStore.EventData","type":"module"},{"doc":"","ref":"Commanded.EventStore.EventData.html#t:t/0","title":"Commanded.EventStore.EventData.t/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.EventData.html#t:uuid/0","title":"Commanded.EventStore.EventData.uuid/0","type":"type"},{"doc":"Contains the persisted stream identity, type, data, and metadata for a single event. Events are immutable once recorded. Recorded event fields event_id - a globally unique UUID to identify the event. event_number - a globally unique, monotonically incrementing and gapless integer used to order the event amongst all events. stream_id - the stream identity for the event. stream_version - the version of the stream for the event. causation_id - an optional UUID identifier used to identify which message you are responding to. correlation_id - an optional UUID identifier used to correlate related messages. data - the event data deserialized into a struct. metadata - a string keyed map of metadata associated with the event. created_at - the datetime, in UTC, indicating when the event was created.","ref":"Commanded.EventStore.RecordedEvent.html","title":"Commanded.EventStore.RecordedEvent","type":"module"},{"doc":"Enrich the event's metadata with fields from the RecordedEvent struct and any additional metadata passed as an option.","ref":"Commanded.EventStore.RecordedEvent.html#enrich_metadata/2","title":"Commanded.EventStore.RecordedEvent.enrich_metadata/2","type":"function"},{"doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:t/0","title":"Commanded.EventStore.RecordedEvent.t/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.RecordedEvent.html#t:uuid/0","title":"Commanded.EventStore.RecordedEvent.uuid/0","type":"type"},{"doc":"Snapshot data","ref":"Commanded.EventStore.SnapshotData.html","title":"Commanded.EventStore.SnapshotData","type":"module"},{"doc":"","ref":"Commanded.EventStore.SnapshotData.html#t:t/0","title":"Commanded.EventStore.SnapshotData.t/0","type":"type"},{"doc":"Specification to convert between an Elixir struct and a corresponding string type.","ref":"Commanded.EventStore.TypeProvider.html","title":"Commanded.EventStore.TypeProvider","type":"behaviour"},{"doc":"Type of the given Elixir struct as a string","ref":"Commanded.EventStore.TypeProvider.html#c:to_string/1","title":"Commanded.EventStore.TypeProvider.to_string/1","type":"callback"},{"doc":"Convert the given type string to an Elixir struct","ref":"Commanded.EventStore.TypeProvider.html#c:to_struct/1","title":"Commanded.EventStore.TypeProvider.to_struct/1","type":"callback"},{"doc":"Get the configured type provider","ref":"Commanded.EventStore.TypeProvider.html#type_provider/0","title":"Commanded.EventStore.TypeProvider.type_provider/0","type":"function"},{"doc":"","ref":"Commanded.EventStore.TypeProvider.html#t:t/0","title":"Commanded.EventStore.TypeProvider.t/0","type":"type"},{"doc":"","ref":"Commanded.EventStore.TypeProvider.html#t:type/0","title":"Commanded.EventStore.TypeProvider.type/0","type":"type"},{"doc":"Use the pubsub configured for a Commanded application.","ref":"Commanded.PubSub.html","title":"Commanded.PubSub","type":"module"},{"doc":"Get the configured pub/sub adapter. Defaults to a local pub/sub, restricted to running on a single node.","ref":"Commanded.PubSub.html#adapter/2","title":"Commanded.PubSub.adapter/2","type":"function"},{"doc":"Broadcasts message on given topic. topic - The topic to broadcast to, ie: &quot;users:123&quot; message - The payload of the broadcast","ref":"Commanded.PubSub.html#broadcast/3","title":"Commanded.PubSub.broadcast/3","type":"function"},{"doc":"Subscribes the caller to the PubSub adapter's topic.","ref":"Commanded.PubSub.html#subscribe/2","title":"Commanded.PubSub.subscribe/2","type":"function"},{"doc":"","ref":"Commanded.PubSub.html#t:application/0","title":"Commanded.PubSub.application/0","type":"type"},{"doc":"","ref":"Commanded.PubSub.html#t:config/0","title":"Commanded.PubSub.config/0","type":"type"},{"doc":"Pub/sub behaviour for use by Commanded to subscribe to and broadcast messages.","ref":"Commanded.PubSub.Adapter.html","title":"Commanded.PubSub.Adapter","type":"behaviour"},{"doc":"Broadcasts message on given topic. topic - The topic to broadcast to, ie: &quot;users:123&quot; message - The payload of the broadcast","ref":"Commanded.PubSub.Adapter.html#c:broadcast/3","title":"Commanded.PubSub.Adapter.broadcast/3","type":"callback"},{"doc":"Return an optional supervisor spec for pub/sub.","ref":"Commanded.PubSub.Adapter.html#c:child_spec/2","title":"Commanded.PubSub.Adapter.child_spec/2","type":"callback"},{"doc":"List tracked PIDs for a given topic.","ref":"Commanded.PubSub.Adapter.html#c:list/2","title":"Commanded.PubSub.Adapter.list/2","type":"callback"},{"doc":"Subscribes the caller to the PubSub adapter's topic.","ref":"Commanded.PubSub.Adapter.html#c:subscribe/2","title":"Commanded.PubSub.Adapter.subscribe/2","type":"callback"},{"doc":"Track the current process under the given topic , uniquely identified by key .","ref":"Commanded.PubSub.Adapter.html#c:track/3","title":"Commanded.PubSub.Adapter.track/3","type":"callback"},{"doc":"","ref":"Commanded.PubSub.Adapter.html#t:adapter_meta/0","title":"Commanded.PubSub.Adapter.adapter_meta/0","type":"type"},{"doc":"","ref":"Commanded.PubSub.Adapter.html#t:application/0","title":"Commanded.PubSub.Adapter.application/0","type":"type"},{"doc":"Local pub/sub adapter, restricted to a single node, using Elixir's Registry . You can configure this adapter in your environment config file: # `config/config.exs` config :my_app , MyApp.Application , pubsub : :local This adapter will be used by default when none is specified in config.","ref":"Commanded.PubSub.LocalPubSub.html","title":"Commanded.PubSub.LocalPubSub","type":"module"},{"doc":"Broadcasts message on given topic.","ref":"Commanded.PubSub.LocalPubSub.html#broadcast/3","title":"Commanded.PubSub.LocalPubSub.broadcast/3","type":"function"},{"doc":"Start a Registry for local pub/sub.","ref":"Commanded.PubSub.LocalPubSub.html#child_spec/2","title":"Commanded.PubSub.LocalPubSub.child_spec/2","type":"function"},{"doc":"List tracked terms and associated PIDs for a given topic.","ref":"Commanded.PubSub.LocalPubSub.html#list/2","title":"Commanded.PubSub.LocalPubSub.list/2","type":"function"},{"doc":"Subscribes the caller to the topic.","ref":"Commanded.PubSub.LocalPubSub.html#subscribe/2","title":"Commanded.PubSub.LocalPubSub.subscribe/2","type":"function"},{"doc":"Track the current process under the given topic , uniquely identified by key .","ref":"Commanded.PubSub.LocalPubSub.html#track/3","title":"Commanded.PubSub.LocalPubSub.track/3","type":"function"},{"doc":"Pub/sub adapter using Phoenix's distributed pub/sub and presence platform. To use Phoenix pub/sub you must add it as a dependency in your project's mix.exs file: defp deps do [ { :phoenix_pubsub , &quot;~&gt; 1.0&quot; } ] end Fetch mix deps and configure the pubsub settings in your environment config file: # `config/config.exs` config :my_app , MyApp.Application , pubsub : [ phoenix_pubsub : [ adapter : Phoenix.PubSub.PG2 , pool_size : 1 ] ] Specify the Phoenix pub/sub adapter you wish to use from: Phoenix.PubSub.PG2 - uses Distributed Elixir, directly exchanging notifications between servers Phoenix.PubSub.Redis - uses Redis to exchange data between servers","ref":"Commanded.PubSub.PhoenixPubSub.html","title":"Commanded.PubSub.PhoenixPubSub","type":"module"},{"doc":"Broadcasts message on given topic.","ref":"Commanded.PubSub.PhoenixPubSub.html#broadcast/3","title":"Commanded.PubSub.PhoenixPubSub.broadcast/3","type":"function"},{"doc":"Start the configured Phoenix pub/sub adapter and a presence tracker.","ref":"Commanded.PubSub.PhoenixPubSub.html#child_spec/2","title":"Commanded.PubSub.PhoenixPubSub.child_spec/2","type":"function"},{"doc":"List tracked terms and associated PIDs for a given topic.","ref":"Commanded.PubSub.PhoenixPubSub.html#list/2","title":"Commanded.PubSub.PhoenixPubSub.list/2","type":"function"},{"doc":"Subscribes the caller to the topic.","ref":"Commanded.PubSub.PhoenixPubSub.html#subscribe/2","title":"Commanded.PubSub.PhoenixPubSub.subscribe/2","type":"function"},{"doc":"Track the current process under the given topic , uniquely identified by key .","ref":"Commanded.PubSub.PhoenixPubSub.html#track/3","title":"Commanded.PubSub.PhoenixPubSub.track/3","type":"function"},{"doc":"Use the process registry configured for a Commanded application.","ref":"Commanded.Registration.html","title":"Commanded.Registration","type":"module"},{"doc":"Allow a registry adapter to handle the standard GenServer callback functions.","ref":"Commanded.Registration.html#__before_compile__/1","title":"Commanded.Registration.__before_compile__/1","type":"macro"},{"doc":"Use the Commanded.Registration module to import the registry adapter and via tuple functions.","ref":"Commanded.Registration.html#__using__/1","title":"Commanded.Registration.__using__/1","type":"macro"},{"doc":"Get the configured process registry. Defaults to a local registry, restricted to running on a single node.","ref":"Commanded.Registration.html#adapter/2","title":"Commanded.Registration.adapter/2","type":"function"},{"doc":"","ref":"Commanded.Registration.html#t:application/0","title":"Commanded.Registration.application/0","type":"type"},{"doc":"","ref":"Commanded.Registration.html#t:config/0","title":"Commanded.Registration.config/0","type":"type"},{"doc":"Defines a behaviour for a process registry to be used by Commanded. By default, Commanded will use a local process registry, defined in Commanded.Registration.LocalRegistry , that uses Elixir's Registry module for local process registration. This limits Commanded to only run on a single node. However the Commanded.Registration behaviour can be implemented by a library to provide distributed process registration to support running on a cluster of nodes.","ref":"Commanded.Registration.Adapter.html","title":"Commanded.Registration.Adapter","type":"behaviour"},{"doc":"Return an optional supervisor spec for the registry","ref":"Commanded.Registration.Adapter.html#c:child_spec/2","title":"Commanded.Registration.Adapter.child_spec/2","type":"callback"},{"doc":"Starts a uniquely named child process of a supervisor using the given module and args. Registers the pid with the given name.","ref":"Commanded.Registration.Adapter.html#c:start_child/4","title":"Commanded.Registration.Adapter.start_child/4","type":"callback"},{"doc":"Starts a uniquely named GenServer process for the given module and args. Registers the pid with the given name.","ref":"Commanded.Registration.Adapter.html#c:start_link/5","title":"Commanded.Registration.Adapter.start_link/5","type":"callback"},{"doc":"Use to start a supervisor.","ref":"Commanded.Registration.Adapter.html#c:supervisor_child_spec/3","title":"Commanded.Registration.Adapter.supervisor_child_spec/3","type":"callback"},{"doc":"Return a :via tuple to route a message to a process by its registered name","ref":"Commanded.Registration.Adapter.html#c:via_tuple/2","title":"Commanded.Registration.Adapter.via_tuple/2","type":"callback"},{"doc":"Get the pid of a registered name. Returns :undefined if the name is unregistered.","ref":"Commanded.Registration.Adapter.html#c:whereis_name/2","title":"Commanded.Registration.Adapter.whereis_name/2","type":"callback"},{"doc":"","ref":"Commanded.Registration.Adapter.html#t:adapter_meta/0","title":"Commanded.Registration.Adapter.adapter_meta/0","type":"type"},{"doc":"","ref":"Commanded.Registration.Adapter.html#t:application/0","title":"Commanded.Registration.Adapter.application/0","type":"type"},{"doc":"","ref":"Commanded.Registration.Adapter.html#t:config/0","title":"Commanded.Registration.Adapter.config/0","type":"type"},{"doc":"","ref":"Commanded.Registration.Adapter.html#t:start_child_arg/0","title":"Commanded.Registration.Adapter.start_child_arg/0","type":"type"},{"doc":"Distributed process registration using Erlangs :global registry[1]. [1] http://erlang.org/doc/man/global.html","ref":"Commanded.Registration.GlobalRegistry.html","title":"Commanded.Registration.GlobalRegistry","type":"module"},{"doc":"Return an optional supervisor spec for the registry","ref":"Commanded.Registration.GlobalRegistry.html#child_spec/2","title":"Commanded.Registration.GlobalRegistry.child_spec/2","type":"function"},{"doc":"Starts a uniquely named child process of a supervisor using the given module and args. Registers the pid with the given name.","ref":"Commanded.Registration.GlobalRegistry.html#start_child/4","title":"Commanded.Registration.GlobalRegistry.start_child/4","type":"function"},{"doc":"Starts a uniquely named GenServer process for the given module and args. Registers the pid with the given name.","ref":"Commanded.Registration.GlobalRegistry.html#start_link/5","title":"Commanded.Registration.GlobalRegistry.start_link/5","type":"function"},{"doc":"Starts a supervisor.","ref":"Commanded.Registration.GlobalRegistry.html#supervisor_child_spec/3","title":"Commanded.Registration.GlobalRegistry.supervisor_child_spec/3","type":"function"},{"doc":"Return a :via tuple to route a message to a process by its registered name","ref":"Commanded.Registration.GlobalRegistry.html#via_tuple/2","title":"Commanded.Registration.GlobalRegistry.via_tuple/2","type":"function"},{"doc":"Get the pid of a registered name. Returns :undefined if the name is unregistered.","ref":"Commanded.Registration.GlobalRegistry.html#whereis_name/2","title":"Commanded.Registration.GlobalRegistry.whereis_name/2","type":"function"},{"doc":"Local process registration, restricted to a single node, using Elixir's Registry module.","ref":"Commanded.Registration.LocalRegistry.html","title":"Commanded.Registration.LocalRegistry","type":"module"},{"doc":"Return a supervisor spec for the registry.","ref":"Commanded.Registration.LocalRegistry.html#child_spec/2","title":"Commanded.Registration.LocalRegistry.child_spec/2","type":"function"},{"doc":"Starts a uniquely named child process of a supervisor using the given module and args. Registers the pid with the given name.","ref":"Commanded.Registration.LocalRegistry.html#start_child/4","title":"Commanded.Registration.LocalRegistry.start_child/4","type":"function"},{"doc":"Starts a uniquely named GenServer process for the given module and args. Registers the pid with the given name.","ref":"Commanded.Registration.LocalRegistry.html#start_link/5","title":"Commanded.Registration.LocalRegistry.start_link/5","type":"function"},{"doc":"Starts a supervisor.","ref":"Commanded.Registration.LocalRegistry.html#supervisor_child_spec/3","title":"Commanded.Registration.LocalRegistry.supervisor_child_spec/3","type":"function"},{"doc":"Return a :via tuple to route a message to a process by its registered name.","ref":"Commanded.Registration.LocalRegistry.html#via_tuple/2","title":"Commanded.Registration.LocalRegistry.via_tuple/2","type":"function"},{"doc":"Get the pid of a registered name. Returns :undefined if the name is unregistered.","ref":"Commanded.Registration.LocalRegistry.html#whereis_name/2","title":"Commanded.Registration.LocalRegistry.whereis_name/2","type":"function"},{"doc":"","ref":"Commanded.Serialization.JsonDecoder.html","title":"Commanded.Serialization.JsonDecoder","type":"protocol"},{"doc":"Protocol to allow additional decoding of a value that has been deserialized using the Commanded.Serialization.JsonSerializer . The protocol is optional. The default behaviour is to to return the value if an explicit protocol is not defined.","ref":"Commanded.Serialization.JsonDecoder.html#decode/1","title":"Commanded.Serialization.JsonDecoder.decode/1","type":"function"},{"doc":"","ref":"Commanded.Serialization.JsonDecoder.html#t:t/0","title":"Commanded.Serialization.JsonDecoder.t/0","type":"type"},{"doc":"A serializer that uses the JSON format and Jason library.","ref":"Commanded.Serialization.JsonSerializer.html","title":"Commanded.Serialization.JsonSerializer","type":"module"},{"doc":"Deserialize given JSON binary data to the expected type.","ref":"Commanded.Serialization.JsonSerializer.html#deserialize/2","title":"Commanded.Serialization.JsonSerializer.deserialize/2","type":"function"},{"doc":"Serialize given term to JSON binary data.","ref":"Commanded.Serialization.JsonSerializer.html#serialize/1","title":"Commanded.Serialization.JsonSerializer.serialize/1","type":"function"},{"doc":"A type provider that uses the Elixir module name Example: %An.Event{} module mapped to &quot;Elixir.An.Event&quot;.","ref":"Commanded.Serialization.ModuleNameTypeProvider.html","title":"Commanded.Serialization.ModuleNameTypeProvider","type":"module"},{"doc":"Callback implementation for Commanded.EventStore.TypeProvider.to_string/1 .","ref":"Commanded.Serialization.ModuleNameTypeProvider.html#to_string/1","title":"Commanded.Serialization.ModuleNameTypeProvider.to_string/1","type":"function"},{"doc":"Callback implementation for Commanded.EventStore.TypeProvider.to_struct/1 .","ref":"Commanded.Serialization.ModuleNameTypeProvider.html#to_struct/1","title":"Commanded.Serialization.ModuleNameTypeProvider.to_struct/1","type":"function"},{"doc":"Reset an event handler. Usage mix commanded . reset -- app &lt; app &gt; -- handler &lt; handler_name &gt; Examples mix commanded . reset -- app MyApp -- handler MyHandler mix commanded . reset - a MyApp - h MyHandler Command line options -a , --app - the Commanded application -h , --handler - the name of the event handler to reset -q , --quiet - do not log output","ref":"Mix.Tasks.Commanded.Reset.html","title":"Mix.Tasks.Commanded.Reset","type":"task"},{"doc":"Middleware provides an extension point to add functions that you want to be called for every command the router dispatches. Examples include command validation, authorization, and logging. Implement the Commanded.Middleware behaviour in your module and define the before_dispatch/1 , after_dispatch/1 , and after_failure/1 callback functions. Example middleware defmodule NoOpMiddleware do @behaviour Commanded.Middleware alias Commanded.Middleware.Pipeline import Pipeline def before_dispatch ( % Pipeline { command : command } = pipeline ) do pipeline end def after_dispatch ( % Pipeline { command : command } = pipeline ) do pipeline end def after_failure ( % Pipeline { command : command } = pipeline ) do pipeline end end Import the Commanded.Middleware.Pipeline module to access convenience functions. assign/3 - puts a key and value into the assigns map halt/1 - stops execution of further middleware downstream and prevents dispatch of the command when used in a before_dispatch callback","ref":"Commanded.Middleware.html","title":"Commanded.Middleware","type":"behaviour"},{"doc":"","ref":"Commanded.Middleware.html#c:after_dispatch/1","title":"Commanded.Middleware.after_dispatch/1","type":"callback"},{"doc":"","ref":"Commanded.Middleware.html#c:after_failure/1","title":"Commanded.Middleware.after_failure/1","type":"callback"},{"doc":"","ref":"Commanded.Middleware.html#c:before_dispatch/1","title":"Commanded.Middleware.before_dispatch/1","type":"callback"},{"doc":"","ref":"Commanded.Middleware.html#t:pipeline/0","title":"Commanded.Middleware.pipeline/0","type":"type"},{"doc":"An internal Commanded.Middleware that blocks after successful command dispatch until the requested dispatch consistency has been met. Only applies when the requested consistency is :strong . Has no effect for :eventual consistency.","ref":"Commanded.Middleware.ConsistencyGuarantee.html","title":"Commanded.Middleware.ConsistencyGuarantee","type":"module"},{"doc":"Callback implementation for Commanded.Middleware.after_dispatch/1 .","ref":"Commanded.Middleware.ConsistencyGuarantee.html#after_dispatch/1","title":"Commanded.Middleware.ConsistencyGuarantee.after_dispatch/1","type":"function"},{"doc":"Callback implementation for Commanded.Middleware.after_failure/1 .","ref":"Commanded.Middleware.ConsistencyGuarantee.html#after_failure/1","title":"Commanded.Middleware.ConsistencyGuarantee.after_failure/1","type":"function"},{"doc":"Callback implementation for Commanded.Middleware.before_dispatch/1 .","ref":"Commanded.Middleware.ConsistencyGuarantee.html#before_dispatch/1","title":"Commanded.Middleware.ConsistencyGuarantee.before_dispatch/1","type":"function"},{"doc":"An internal Commanded.Middleware that extracts the target aggregate's identity from the command.","ref":"Commanded.Middleware.ExtractAggregateIdentity.html","title":"Commanded.Middleware.ExtractAggregateIdentity","type":"module"},{"doc":"Callback implementation for Commanded.Middleware.after_dispatch/1 .","ref":"Commanded.Middleware.ExtractAggregateIdentity.html#after_dispatch/1","title":"Commanded.Middleware.ExtractAggregateIdentity.after_dispatch/1","type":"function"},{"doc":"Callback implementation for Commanded.Middleware.after_failure/1 .","ref":"Commanded.Middleware.ExtractAggregateIdentity.html#after_failure/1","title":"Commanded.Middleware.ExtractAggregateIdentity.after_failure/1","type":"function"},{"doc":"Callback implementation for Commanded.Middleware.before_dispatch/1 .","ref":"Commanded.Middleware.ExtractAggregateIdentity.html#before_dispatch/1","title":"Commanded.Middleware.ExtractAggregateIdentity.before_dispatch/1","type":"function"},{"doc":"A Commanded.Middleware that logs each stage of the command dispatch using the Elixir Logger : Before dispatch. After successful dispatch. After failed dispatch.","ref":"Commanded.Middleware.Logger.html","title":"Commanded.Middleware.Logger","type":"module"},{"doc":"Callback implementation for Commanded.Middleware.after_dispatch/1 .","ref":"Commanded.Middleware.Logger.html#after_dispatch/1","title":"Commanded.Middleware.Logger.after_dispatch/1","type":"function"},{"doc":"Callback implementation for Commanded.Middleware.after_failure/1 .","ref":"Commanded.Middleware.Logger.html#after_failure/1","title":"Commanded.Middleware.Logger.after_failure/1","type":"function"},{"doc":"Callback implementation for Commanded.Middleware.before_dispatch/1 .","ref":"Commanded.Middleware.Logger.html#before_dispatch/1","title":"Commanded.Middleware.Logger.before_dispatch/1","type":"function"},{"doc":"Pipeline is a struct used as an argument in the callback functions of modules implementing the Commanded.Middleware behaviour. This struct must be returned by each function to be used in the next middleware based on the configured middleware chain. Pipeline fields application - the Commanded application. assigns - shared user data as a map. causation_id - an optional UUID used to identify the cause of the command being dispatched. correlation_id - an optional UUID used to correlate related commands/events together. command - command struct being dispatched. command_uuid - UUID assigned to the command being dispatched. consistency - requested dispatch consistency, either: :eventual (default) or :strong . halted - flag indicating whether the pipeline was halted. identity - an atom specifying a field in the command containing the aggregate's identity or a one-arity function that returns an identity from the command being dispatched. identity_prefix - an optional prefix to the aggregate's identity. It may be a string (e.g. &quot;prefix-&quot;) or a zero arity function (e.g. &amp;MyRouter.identity_prefix/0 ). metadata - the metadata map to be persisted along with the events. response - sets the response to send back to the caller.","ref":"Commanded.Middleware.Pipeline.html","title":"Commanded.Middleware.Pipeline","type":"module"},{"doc":"Puts the key with value equal to value into assigns map.","ref":"Commanded.Middleware.Pipeline.html#assign/3","title":"Commanded.Middleware.Pipeline.assign/3","type":"function"},{"doc":"Puts the key with value equal to value into metadata map. Note: Use of atom keys in metadata is deprecated in favour of binary strings.","ref":"Commanded.Middleware.Pipeline.html#assign_metadata/3","title":"Commanded.Middleware.Pipeline.assign_metadata/3","type":"function"},{"doc":"Executes the middleware chain.","ref":"Commanded.Middleware.Pipeline.html#chain/3","title":"Commanded.Middleware.Pipeline.chain/3","type":"function"},{"doc":"Halts the pipeline by preventing further middleware downstream from being invoked. Prevents dispatch of the command if halt occurs in a before_dispatch callback.","ref":"Commanded.Middleware.Pipeline.html#halt/1","title":"Commanded.Middleware.Pipeline.halt/1","type":"function"},{"doc":"Has the pipeline been halted?","ref":"Commanded.Middleware.Pipeline.html#halted?/1","title":"Commanded.Middleware.Pipeline.halted?/1","type":"function"},{"doc":"Sets the response to be returned to the dispatch caller, unless already set.","ref":"Commanded.Middleware.Pipeline.html#respond/2","title":"Commanded.Middleware.Pipeline.respond/2","type":"function"},{"doc":"Extract the response from the pipeline","ref":"Commanded.Middleware.Pipeline.html#response/1","title":"Commanded.Middleware.Pipeline.response/1","type":"function"},{"doc":"Provides test assertion and wait for event functions to help test applications built using Commanded. The default assert and refute receive timeouts are one second. You can override the default timeout in config (e.g. config/test.exs ): config :commanded , assert_receive_event_timeout : 1_000 , refute_receive_event_timeout : 1_000","ref":"Commanded.Assertions.EventAssertions.html","title":"Commanded.Assertions.EventAssertions","type":"module"},{"doc":"Assert that events matching their respective predicates have a matching correlation id. Useful when there is a chain of events that is connected through event handlers. Example assert_correlated ( BankApp , BankAccountOpened , fn opened -&gt; opened . id == 1 end , InitialAmountDeposited , fn deposited -&gt; deposited . id == 2 end )","ref":"Commanded.Assertions.EventAssertions.html#assert_correlated/5","title":"Commanded.Assertions.EventAssertions.assert_correlated/5","type":"function"},{"doc":"Assert that an event of the given event type is published. Verify that event using the assertion function. Example assert_receive_event ( BankApp , BankAccountOpened , fn opened -&gt; assert opened . account_number == &quot;ACC123&quot; end )","ref":"Commanded.Assertions.EventAssertions.html#assert_receive_event/3","title":"Commanded.Assertions.EventAssertions.assert_receive_event/3","type":"function"},{"doc":"Assert that an event of the given event type, matching the predicate, is published. Verify that event using the assertion function. Example assert_receive_event ( BankApp , BankAccountOpened , fn opened -&gt; opened . account_number == &quot;ACC123&quot; end , fn opened -&gt; assert opened . balance == 1_000 end )","ref":"Commanded.Assertions.EventAssertions.html#assert_receive_event/4","title":"Commanded.Assertions.EventAssertions.assert_receive_event/4","type":"function"},{"doc":"Refute that an event of the given type has been received. An optional predicate may be provided to filter events matching the refuted type. Examples Refute that ExampleEvent is produced by given anonymous function: refute_receive_event(ExampleApp, ExampleEvent, fn -&gt; :ok = MyApp . dispatch ( command ) end) Refute that ExampleEvent is produced by some_func/0 function: refute_receive_event(ExampleApp, ExampleEvent, &amp;some_func/0) Refute that ExampleEvent matching given event_matches?/1 predicate function is produced by some_func/0 function: refute_receive_event ( ExampleApp , ExampleEvent , &amp; some_func / 0 , predicate : &amp; event_matches? / 1 ) Refute that ExampleEvent matching given anonymous predicate function is produced by some_func/0 function: refute_receive_event ( ExampleApp , ExampleEvent , &amp; some_func / 0 , predicate : fn event -&gt; event . value == 1 end ) Refute that ExampleEvent produced by some_func/0 function is published to a given stream: refute_receive_event ( ExampleApp , ExampleEvent , &amp; some_func / 0 , predicate : fn event -&gt; event . value == 1 end , stream : &quot;foo-1234&quot; )","ref":"Commanded.Assertions.EventAssertions.html#refute_receive_event/4","title":"Commanded.Assertions.EventAssertions.refute_receive_event/4","type":"function"},{"doc":"Wait for an event of the given event type to be published. Examples wait_for_event ( BankApp , BankAccountOpened )","ref":"Commanded.Assertions.EventAssertions.html#wait_for_event/2","title":"Commanded.Assertions.EventAssertions.wait_for_event/2","type":"function"},{"doc":"Wait for an event of the given event type, matching the predicate, to be published. Examples wait_for_event ( BankApp , BankAccountOpened , fn opened -&gt; opened . account_number == &quot;ACC123&quot; end )","ref":"Commanded.Assertions.EventAssertions.html#wait_for_event/3","title":"Commanded.Assertions.EventAssertions.wait_for_event/3","type":"function"},{"doc":"","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"Allow a process manager to stop after dispatching a command ( #460 ). Replace use Mix.Config with import Config in config files ( #467 ). Event handler concurrency ( #486 ).","ref":"changelog.html#v1-4-0-rc-0","title":"Changelog - v1.4.0-rc.0","type":"extras"},{"doc":"Bug fixes Event Handler not calling init/1 callback function on restart ( #463 ). Call process manager init/1 function on process restart ( #464 ).","ref":"changelog.html#v1-3-1","title":"Changelog - v1.3.1","type":"extras"},{"doc":"Enhancements Allow command identity to be provided during dispatch ( #406 ). Define Commanded.Telemetry module to emit consistent telemetry events ( #414 ). Telemetry [:commanded, :aggregate, :execute] events ( #407 ). Telemetry [:commanded, :event, :handle] events ( #408 ). Telemetry [:commanded, :process_manager, :handle] events ( #418 ). Telemetry [:commanded, :application, :dispatch] ( #423 ). Graceful shutdown of event handlers ( #431 ). Ensure command dispatch metadata is a map ( #432 ). Retry command execution on node down ( #429 ). Dispatch returning resultant events ( #444 ). Get aggregate state ( #448 ). Support telemetry v1.0 ( #456 ).","ref":"changelog.html#v1-3-0","title":"Changelog - v1.3.0","type":"extras"},{"doc":"Enhancements Add init/1 callback function to event handlers and process managers ( #393 ). Include application and handler_name as additional event handler metadata ( #396 ). Allow GenServer start options to be provided when starting event handlers and process managers ( #398 ). Add hibernate_after option to application config ( #399 ). Add support for providing adapter-specific event store subscription options ( #391 ). Support custom state for event handlers ( #400 ). Allow event handlers and process manager error callback to return failure context struct ( #397 ). Allow a before execute function to be defined which is called with the command dispatch execution context and aggregate state before ( #402 ). Bug fixes Allow process manager error/3 callback to return :skip for failed commands, not just failed events ( #362 ).","ref":"changelog.html#v1-2-0","title":"Changelog - v1.2.0","type":"extras"},{"doc":"Enhancements Capture exception on Process Manager apply/2 and call error/3 callback functions ( #380 ). Include metadata in upcaster protocol ( #389 ). Bug fixes Fix Commanded.Aggregate.Multi.execute/2 calls which return Multi struct ( #385 ).","ref":"changelog.html#v1-1-1","title":"Changelog - v1.1.1","type":"extras"},{"doc":"Enhancements Dynamic Commanded applications ( #324 ). Log and ignore unexpected messages received by event handlers and process manager instances ( #333 ) Process manager identity/0 function ( #334 ). Extend Commanded.AggregateCase ExUnit case template to support Commanded.Aggregate.Multi . Allow Commanded.Aggregate.Multi to return events as :ok tagged tuples. Run the formatter in CI ( #341 ). Add stacktraces to EventHandler error logging ( #340 ) refute_receive_event/4 only tests newly created events ( #347 ). Allow Commanded Application name to be set dynamically in middleware ( #352 ). Remove router module compile-time checking ( #363 ). Reduce memory consumption during aggregate state rebuild ( #368 ). Upgrade to phoenix_pubsub to 2.0 ( #365 ). Ignore :not_found error when resetting InMemory event store ( #354 ). Add router/1 to locals_without_parens in Mix format config ( #351 ). Include stacktrace in event handler and process manager error callback functions ( #342 ). Call event handler's error/3 callback function when handle/2 function returns an invalid value ( #372 ). Bug fixes Fixes the typespec for command dispatch ( #325 ). Process manager stops if interested?/1 returns an empty list ( #335 ).","ref":"changelog.html#v1-1-0","title":"Changelog - v1.1.0","type":"extras"},{"doc":"Enhancements Global registry using Erlang's :global module ( #344 ). Command dispatch return ( #331 ). Bug fixes Fix distributed subscription registration bug ( #345 ). Retry event handler and process manager subscriptions on error ( #348 ).","ref":"changelog.html#v1-0-1","title":"Changelog - v1.0.1","type":"extras"},{"doc":"Breaking changes Support multiple Commanded apps ( #298 ). Enhancements Define adapter behaviour modules for event store, pubsub, and registry ( #311 ). Add AggregateCase ExUnit case template to support aggregate unit testing ( #315 ). Application config lookup ( #318 ). Bug fixes Fix process manager exception on start ( #307 ). Fix commanded aggregate race ( #308 ). Fix Dialyzer warnings and include in Travis CI ( #317 ). Upgrading Follow the upgrade guide to define and use your own Commanded application.","ref":"changelog.html#v1-0-0","title":"Changelog - v1.0.0","type":"extras"},{"doc":"Enhancements Reset event handler mix task mix commanded.reset MyApp.Handler ( #293 ). Bug fixes Fix regression in Commanded.Middleware.Logger.delta ( #295 ).","ref":"changelog.html#v0-19-1","title":"Changelog - v0.19.1","type":"extras"},{"doc":"Enhancements Update typespec for data and metadata fields in Commanded.EventStore.EventData struct ( #246 ). Add include_execution_result and aggregate_version to typespec for router command dispatch ( #262 ). Add .formatter.exs to Hex package ( #247 ). Event upcasting ( #263 ). Support :ok tagged tuple events from aggregate ( #268 ). Modify Commanded.Registration.start_child to pass a child_spec ( #273 ). Add supervisor_child_spec/2 to Commanded.Registration behaviour ( #277 ) used by Commanded Horde Registry . Ensure Commanded can be compiled when optional Jason dependency is not present ( #286 ). Fix Aggregate initialization races ( #287 ). Support {:system, varname} format in Phoenix PubSub config ( #291 ). Breaking changes Use DateTime instead of NaiveDateTime for all datetimes ( #254 ). This affects the created_at field defined in the Commanded.EventStore.RecordedEvent . You will need to migrate from NaiveDateTime to DateTime if you use this field in your code (such as in an event handler's metadata).","ref":"changelog.html#v0-19-0","title":"Changelog - v0.19.0","type":"extras"},{"doc":"Enhancements Process manager idle process timeout ( #290 ). Register event handler and process manager subscriptions on process start ( #272 ).","ref":"changelog.html#v0-18-1","title":"Changelog - v0.18.1","type":"extras"},{"doc":"Enhancements Rename uuid dependency to elixir_uuid ( #178 ). Allow aggregate identity to be of any type that implements the String.Chars protocol ( #166 ). Process manager and event handler error &amp; exception handling ( #192 ). Process manager event handling timeout ( #193 ). Allow event handlers to subscribe to individual streams ( #203 ). Add new values for expected_version for event store append events behaviour ( #127 ). Export Commanded.Commands.Router macros in .formatter.exs file ( #204 ). Generate specs and docs for Router dispatch functions only once ( #206 ). Allow two-arity predicate function in wait_for_event receiving both event data and recorded event struct ( #213 ). Allow :infinity timeout on command dispatch ( #227 ) Strict process manager routing ( #243 ). Allow Commanded.Aggregate.Multi to be nested ( #244 ). Add child_spec/0 function to Commanded.EventStore behaviour. Add delete_subscription/2 to Commanded.EventStore behaviour ( #245 ). Add refute_receive_event/2 to Commanded.Assertions.EventAssertions test helpers. Bug fixes Fix typo in include_execution_result global router option ( #216 ). Handle the {:ok, _} tuple dispatch result in process manager command dispatch ( #236 ). Allow string keys for Commanded.Middleware.Pipeline.assign_metadata/3 , atoms are being deprecated ( #228 ) Fix Commanded.PubSub.subscribe typespec ( #222 ). Breaking changes Migrate to Jason for JSON serialization ( #234 ). You will need to add Jason as a dependency in mix.exs : defp deps do [ { :jason , &quot;~&gt; 1.2&quot; } ] end Jason has no support for encoding arbitrary structs - explicit implementation of the Jason.Encoder protocol is always required. You must update all your domain event modules, aggregate state (when using state snapshotting), and process manager state to include @derive Jason.Encoder as shown below: defmodule AnEvent do @derive Jason.Encoder defstruct [ :field ] end Extend aggregate lifespan behaviour to include after_error/1 and after_command/1 callbacks ( #210 ). Previously you only had to define an after_event/1 callback function to implement the Commanded.Aggregates.AggregateLifespan behaviour: defmodule BankAccountLifespan do @behaviour Commanded.Aggregates.AggregateLifespan def after_event ( % BankAccountClosed { } ) , do : :stop def after_event ( _event ) , do : :infinity end Now you must also define after_command/1 and after_error/1 callback functions: defmodule BankAccountLifespan do @behaviour Commanded.Aggregates.AggregateLifespan def after_event ( % BankAccountClosed { } ) , do : :stop def after_event ( _event ) , do : :infinity def after_command ( % CloseAccount { } ) , do : :stop def after_command ( _command ) , do : :infinity def after_error ( :invalid_initial_balance ) , do : :stop def after_error ( _error ) , do : :stop end Upgrading Please ensure you upgrade the following event store dependencies. Using the Elixir EventStore : eventstore to v0.16.0 commanded_eventstore_adapter to v0.5.0 Using Greg Young's Event Store : commanded_extreme_adapter to v0.6.0 Commanded Ecto projections: commanded_ecto_projections to v0.8.0 Commanded scheduler: commanded_scheduler to v0.2.0","ref":"changelog.html#v0-18-0","title":"Changelog - v0.18.0","type":"extras"},{"doc":"Enhancements Process manager idle process timeout ( #290 ).","ref":"changelog.html#v0-17-5","title":"Changelog - v0.17.5","type":"extras"},{"doc":"Bug fixes Register event handler and process manager subscriptions on process start ( #272 ).","ref":"changelog.html#v0-17-4","title":"Changelog - v0.17.4","type":"extras"},{"doc":"Bug fixes Fix snapshot recording ( #196 ). Fixed warning about deprecated time unit in elixir 1.8 ( #239 ).","ref":"changelog.html#v0-17-3","title":"Changelog - v0.17.3","type":"extras"},{"doc":"Enhancements Remove default error/4 callback function from process manager to silence deprecation warning.","ref":"changelog.html#v0-17-2","title":"Changelog - v0.17.2","type":"extras"},{"doc":"Enhancements Support Phoenix.PubSub v1.1.0. Bug fixes Set default aggregate lifespan timeout to :infinity ( #200 ).","ref":"changelog.html#v0-17-1","title":"Changelog - v0.17.1","type":"extras"},{"doc":"Enhancements Ability to globally override include_execution_result and include_aggregate_version in environment config ( #168 ). Handle custom type serialization in snapshot source type ( #165 ). Fix compiler warnings in generated code (routers, event handlers, and process managers). Add InMemory.reset!/0 for testing purposes ( #175 ). Bug fixes Ensure process managers can be configured with :strong consistency. Fix error when subscription process already tracked ( #180 ).","ref":"changelog.html#v0-17-0","title":"Changelog - v0.17.0","type":"extras"},{"doc":"Support composite command routers ( #111 ). Aggregate state snapshots ( #121 ). New error/3 callback for process manager and deprecated error/4 ( #124 ) Router support for identity prefix function. Retry command execution on concurrency error ( #132 ). Event handler error/3 callback ( #133 ). Support distributed dispatch consistency ( #135 ). Defer event handler and process router init until after subscribed ( #138 ). Replace aggregate lifespan after_command/1 callback with after_event/1 ( #139 ). Support process manager routing to multiple instances ( #141 ). Allow a default consistency to be set via the application env ( #150 ). Command dispatch consistency using explicit handler names ( #161 ). Breaking changes The Commanded.Aggregates.AggregateLifespan behaviour has been changed from after_command/1 to after_event/1 . You will need to update your own lifespan modules to use events instead of commands to shutdown an aggregate process after an inactivity timeout. Upgrading Please ensure you upgrade the following event store dependencies. Using the Elixir EventStore : eventstore to v0.14.0 commanded_eventstore_adapter to v0.4.0 Using Greg Young's Event Store : commanded_extreme_adapter to v0.5.0","ref":"changelog.html#v0-16-0","title":"Changelog - v0.16.0","type":"extras"},{"doc":"Bug fixes Event handler child_spec/1 must include config options defined by use macro.","ref":"changelog.html#v0-15-1","title":"Changelog - v0.15.1","type":"extras"},{"doc":"Enhancements Process manager command dispatch error handling ( #93 ). Event handlers may define an init/0 callback function to start any related processes. It must return :ok , otherwise the handler process will be stopped. Add include_execution_result option to command dispatch ( #96 ). Add Commanded.Aggregate.Multi ( #98 ) as a way to return multiple events from a command dispatch that require aggregate state to be updated after each event. Correlation and causation ids ( #105 ). Initial support for running on a cluster of nodes ( #80 ). Bug fixes Adding a prefix to the aggregate in the router breaks the strong consistency of command dispatch ( #101 ). Upgrading Please ensure you upgrade the following event store dependencies. Using the Elixir EventStore : eventstore to v0.13.0 commanded_eventstore_adapter to v0.3.0 Using Greg's Event Store : commanded_extreme_adapter to v0.4.0","ref":"changelog.html#v0-15-0","title":"Changelog - v0.15.0","type":"extras"},{"doc":"Enhancements Dispatch command with :eventual or :strong consistency guarantee ( #82 ). Additional stream prefix per aggregate ( #77 ). Include custom metadata during command dispatch ( #61 ). Validate command dispatch registration in router ( 59 ). Upgrading Please ensure you upgrade the following event store dependencies. Using the Elixir EventStore : eventstore to v0.11.0 commanded_eventstore_adapter to v0.2.0 Using Greg's Event Store : commanded_extreme_adapter to v0.3.0","ref":"changelog.html#v0-14-0","title":"Changelog - v0.14.0","type":"extras"},{"doc":"Enhancements Command dispatch optionally returns aggregate version, using include_aggregate_version: true during dispatch.","ref":"changelog.html#v0-13-0","title":"Changelog - v0.13.0","type":"extras"},{"doc":"Enhancements Commanded.Event.Handler and Commanded.ProcessManagers.ProcessManager macros to simplify defining, and starting, event handlers and process managers. Note the previous approach to defining and starting may still be used, so this is not a breaking change.","ref":"changelog.html#v0-12-0","title":"Changelog - v0.12.0","type":"extras"},{"doc":"Enhancements Shutdown idle aggregate processes ( #43 ).","ref":"changelog.html#v0-11-0","title":"Changelog - v0.11.0","type":"extras"},{"doc":"Enhancements Extract event store integration to a behaviour ( Commanded.EventStore ). This defines the contract to be implemented by an event store adapter. It allows additional event store databases to be used with Commanded. By default, a GenServer in-memory event store adapter is used. This should only be used for testing as there is no persistence. The existing PostgreSQL-based eventstore integration has been extracted as a separate package ( commanded_eventstore_adapter ). There is also a new adapter for Greg Young's Event Store using the Extreme library ( commanded_extreme_adapter ). You must install the required event store adapter package and update your environment configuration to specify the :event_store_adapter module. See the README for details.","ref":"changelog.html#v0-10-0","title":"Changelog - v0.10.0","type":"extras"},{"doc":"Enhancements Stream events from event store when rebuilding aggregate state.","ref":"changelog.html#v0-9-0","title":"Changelog - v0.9.0","type":"extras"},{"doc":"Enhancements Upgrade to Elixir 1.4 and remove compiler warnings.","ref":"changelog.html#v0-8-5","title":"Changelog - v0.8.5","type":"extras"},{"doc":"Enhancements Event handler and process manager subscriptions should be created from a given stream position ( #14 ). Stop process manager instance after reaching its final state ( #24 ).","ref":"changelog.html#v0-8-4","title":"Changelog - v0.8.4","type":"extras"},{"doc":"Enhancements Middleware after_failure callback is executed even when a middleware halts execution.","ref":"changelog.html#v0-8-3","title":"Changelog - v0.8.3","type":"extras"},{"doc":"Bug fixes JsonSerializer should ensure event type atom exists when deserializing ( #28 ).","ref":"changelog.html#v0-8-2","title":"Changelog - v0.8.2","type":"extras"},{"doc":"Enhancements Command handlers should be optional by default ( #30 ).","ref":"changelog.html#v0-8-1","title":"Changelog - v0.8.1","type":"extras"},{"doc":"Enhancements Simplify aggregates and process managers ( #31 ).","ref":"changelog.html#v0-8-0","title":"Changelog - v0.8.0","type":"extras"},{"doc":"Bug fixes Restarting aggregate process should load all events from its stream in batches. The Event Store read stream default limit is 1,000 events.","ref":"changelog.html#v0-7-1","title":"Changelog - v0.7.1","type":"extras"},{"doc":"Enhancements Command handling middleware allows a command router to define middleware modules that are executed before, and after success or failure of each command dispatch ( #12 ).","ref":"changelog.html#v0-7-0","title":"Changelog - v0.7.0","type":"extras"},{"doc":"Enhancements Process manager instance processes event non-blocking to prevent timeout during event processing and any command dispatching. It persists last seen event id to ensure events are handled only once.","ref":"changelog.html#v0-6-3","title":"Changelog - v0.6.3","type":"extras"},{"doc":"Enhancements Command dispatch timeout. Allow a timeout value to be configured during command registration or dispatch. This overrides the default timeout of 5 seconds. The same as the default GenServer call timeout. Bug fixes Fix pending aggregates restarts: supervisor restarts aggregate process but it cannot accept commands ( #22 ).","ref":"changelog.html#v0-6-2","title":"Changelog - v0.6.2","type":"extras"},{"doc":"Enhancements Upgrade eventstore mix dependency to v0.6.0 to use support for recorded events created_at as NaiveDateTime .","ref":"changelog.html#v0-6-1","title":"Changelog - v0.6.1","type":"extras"},{"doc":"Enhancements Confirm receipt of events in event handler and process manager router ( #19 ). Convert keys to atoms when decoding JSON using Poison decoder. Prefix process manager instance snapshot uuid with process manager name. Multi command dispatch registration in router ( #16 ).","ref":"changelog.html#v0-6-0","title":"Changelog - v0.6.0","type":"extras"},{"doc":"Enhancements Include event metadata as second argument to event handlers. An event handler must now implement the Commanded.Event.Handler behaviour consisting of a single handle_event/2 function.","ref":"changelog.html#v0-5-0","title":"Changelog - v0.5.0","type":"extras"},{"doc":"Enhancements Macro to assist with building process managers ( README ).","ref":"changelog.html#v0-4-0","title":"Changelog - v0.4.0","type":"extras"},{"doc":"Enhancements Include unit test event assertion function: assert_receive_event/2 ( #13 ). Include top level application in mix config.","ref":"changelog.html#v0-3-1","title":"Changelog - v0.3.1","type":"extras"},{"doc":"Enhancements Don't persist an aggregate's pending events when executing a command returns an error ( #10 ). Bug fixes Ensure an aggregate's pending events are persisted in the order they were applied.","ref":"changelog.html#v0-3-0","title":"Changelog - v0.3.0","type":"extras"},{"doc":"Enhancements Support integer, atom or strings as an aggregate UUID ( #7 ).","ref":"changelog.html#v0-2-1","title":"Changelog - v0.2.1","type":"extras"},{"doc":"Initial release.","ref":"changelog.html#v0-1-0","title":"Changelog - v0.1.0","type":"extras"},{"doc":"Commanded can be installed from the package manager hex as follows. Add commanded to your list of dependencies in mix.exs : def deps do [ { :commanded , &quot;~&gt; 1.3&quot; } ] end Optionally add jason to support JSON serialization via Commanded.Serialization.JsonSerializer : def deps do [ { :jason , &quot;~&gt; 1.2&quot; } ] end Fetch mix dependencies: $ mix deps.get Define a Commanded application module for your app, see the Application guide for details. defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app end Configure one of the supported event stores by following the Choosing an Event Store guide .","ref":"getting-started.html","title":"Getting started","type":"extras"},{"doc":"You must decide which event store to use with Commanded. You have a choice between two existing event stores: PostgreSQL-based Elixir EventStore ( adapter ). EventStoreDB ( adapter ). There is also an in-memory event store adapter for test use only . Want to use a different event store? Then you will need to write your own event store provider as described below.","ref":"choosing-an-event-store.html","title":"Choosing an event store","type":"extras"},{"doc":"Use :commanded_eventstore_adapter to persist events to a PostgreSQL database. As the name implies, this is the adapter for EventStore , which is open-source event store using PostgreSQL for persistence and implemented in Elixir.","ref":"choosing-an-event-store.html#postgresql-based-elixir-eventstore","title":"Choosing an event store - PostgreSQL-based Elixir EventStore","type":"extras"},{"doc":"Use :commanded_extreme_adapter to persist events to EventStoreDB : an open-source database, the best data storage solution for event-sourced systems. It can be run as a cluster of nodes containing the same data, which remains available for writes provided at least half the nodes are alive and connected. The quickest way to get started with EventStoreDB is by using their official EventStoreDB Docker container . The Commanded adapter uses the :extreme Elixir TCP client to connect to EventStoreDB. Running EventStoreDB You must run EventStoreDB with all projections enabled and standard projections started. Use the --run-projections=all --start-standard-projections=true flags when running the EventStoreDB executable.","ref":"choosing-an-event-store.html#eventstoredb","title":"Choosing an event store - EventStoreDB","type":"extras"},{"doc":"To use an alternative event store with Commanded you will need to implement the Commanded.EventStore.Adapter behaviour. This defines the contract to be implemented by an adapter module to allow an event store to be used with Commanded. Tests to verify an adapter conforms to the behaviour are provided in test/event_store_adapter . You can use one of the existing adapters ( commanded_eventstore_adapter or commanded_extreme_adapter ) to understand what is required.","ref":"choosing-an-event-store.html#writing-your-own-event-store-provider","title":"Choosing an event store - Writing your own event store provider","type":"extras"},{"doc":"Commanded provides the building blocks for you to create your own Elixir applications following the CQRS/ES pattern. A separate guide is provided for each of the components you can build: Application Aggregates Commands, registration and dispatch Events and handlers Process managers Commanded uses strong consistency for command dispatch (write model) and eventual consistency, by default, for the read model. Receiving an :ok reply from dispatch indicates the command was successfully handled and any created domain events fully persisted to your chosen event store. You may opt into strong consistency for individual event handlers and command dispatch as required.","ref":"usage.html","title":"Using Commanded","type":"extras"},{"doc":"Here's an example bank account opening feature built using Commanded to demonstrate its usage. Define an OpenBankAccount command: defmodule OpenBankAccount do defstruct [ :account_number , :initial_balance ] end Define a corresponding BankAccountOpened domain event: defmodule BankAccountOpened do @derive Jason.Encoder defstruct [ :account_number , :initial_balance ] end Build a BankAccount aggregate to handle the command, protect its business invariants, and return a domain event when successfully handled: defmodule BankAccount do defstruct [ :account_number , :balance ] # Public command API def execute ( % BankAccount { account_number : nil } , % OpenBankAccount { account_number : account_number , initial_balance : initial_balance } ) when initial_balance &gt; 0 do % BankAccountOpened { account_number : account_number , initial_balance : initial_balance } end # Ensure initial balance is never zero or negative def execute ( % BankAccount { } , % OpenBankAccount { initial_balance : initial_balance } ) when initial_balance &lt;= 0 do { :error , :initial_balance_must_be_above_zero } end # Ensure account has not already been opened def execute ( % BankAccount { } , % OpenBankAccount { } ) do { :error , :account_already_opened } end # State mutators def apply ( % BankAccount { } = account , % BankAccountOpened { } = event ) do % BankAccountOpened { account_number : account_number , initial_balance : initial_balance } = event % BankAccount { account | account_number : account_number , balance : initial_balance } end end Define a router module to route the open account command to the bank account aggregate: defmodule BankRouter do use Commanded.Commands.Router dispatch OpenBankAccount , to : BankAccount , identity : :account_number end Define an application to host the aggregate and supporting processes: defmodule BankApp do use Commanded.Application , otp_app : :bank , event_store : [ adapter : Commanded.EventStore.Adapters.InMemory ] router BankRouter end This application is configured to use in-memory event store included with Commanded for testing. Create an event handler module that updates a bank account balance: defmodule AccountBalanceHandler do use Commanded.Event.Handler , application : BankApp , name : __MODULE__ def init do with { :ok , _pid } &lt;- Agent . start_link ( fn -&gt; 0 end , name : __MODULE__ ) do :ok end end def handle ( % BankAccountOpened { initial_balance : initial_balance } , _metadata ) do Agent . update ( __MODULE__ , fn _ -&gt; initial_balance end ) end def current_balance do Agent . get ( __MODULE__ , fn balance -&gt; balance end ) end end Start the application and event handler processes: { :ok , _pid } = BankApp . start_link ( ) { :ok , _pid } = AccountBalanceHandler . start_link ( ) In a real application you would use a supervisor to start these processes. Finally, we can dispatch a command to open a new bank account: :ok = BankApp . dispatch ( % OpenBankAccount { account_number : &quot;ACC123456&quot; , initial_balance : 1_000 } )","ref":"usage.html#quick-overview","title":"Using Commanded - Quick overview","type":"extras"},{"doc":"Commanded allows you to define, supervise, and start your own application module. To use Commanded you must create at least one application. You can create multiple Commanded applications which will run independently, each using its own separately configured event store. The application expects at least an :otp_app option to be specified. It should point to an OTP application containing the application's configuration. For example, the application: defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app router MyApp.Router end Could be configured with: # config/config.exs config :my_app , MyApp.Application , event_store : [ adapter : Commanded.EventStore.Adapters.EventStore , event_store : MyApp.EventStore ] , pubsub : :local , registry : :local Alternatively, you can include the event store, pubsub, and registry config when defining the application: defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app , event_store : [ adapter : Commanded.EventStore.Adapters.EventStore , event_store : MyApp.EventStore ] , pubsub : :local , registry : :local router ( MyApp.Router ) end Finally, you can provide an optional init/1 function to provide runtime configuration. defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app def init ( config ) do { :ok , config } end end","ref":"application.html","title":"Application","type":"extras"},{"doc":"A Commanded application is also a composite router. This provides the router macro allowing you to include one or more router modules for command dispatch. defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app router ( MyApp.Billing.Router ) router ( MyApp.Customers.Router ) router ( MyApp.Notifications.Router ) end Once you have defined a router you can dispatch a command using the application module: :ok = MyApp.Application . dispatch ( % RegisterCustomer { id : UUID . uuid4 ( ) , name : &quot;Ben&quot; } )","ref":"application.html#routing-commands","title":"Application - Routing commands","type":"extras"},{"doc":"Build your aggregates using standard Elixir modules and functions, with structs to hold state. There is no external dependency requirement. An aggregate is comprised of its state, public command functions, and state mutators. In a CQRS application all state must be derived from the published domain events. This prevents tight coupling between aggregate instances, such as querying for their state, and ensures their state isn't exposed. If an aggregate needs data owned by another aggregate, then you should lookup the data from a projection and include it in the command before dispatch. Usually this would be a projection into a read model built for querying. You can use Commanded Ecto projections to project events into a SQL database.","ref":"aggregates.html","title":"Aggregates","type":"extras"},{"doc":"Use the defstruct keyword to define the aggregate state and fields. defmodule ExampleAggregate do defstruct [ :uuid , :name ] end","ref":"aggregates.html#aggregate-state","title":"Aggregates - Aggregate state","type":"extras"},{"doc":"A command function receives the aggregate's state and the command to execute. It must return the resultant domain events, which may be one event or multiple events. You can return a single event or a list of events: %Event{} , [%Event{}] , {:ok, %Event{}} , or {:ok, [%Event{}]} . To respond without returning an event you can return :ok , nil or an empty list as either [] or {:ok, []} . For business rule violations and errors you may return an {:error, error} tagged tuple or raise an exception. Name your public command functions execute/2 to dispatch commands directly to the aggregate without requiring an intermediate command handler. defmodule ExampleAggregate do def execute ( % ExampleAggregate { uuid : nil } , % Create { } = command ) do % Create { uuid : uuid , name : name } = command event = % Created { uuid : uuid , name : name } { :ok , event } end def execute ( % ExampleAggregate { } , % Create { } ) , do : { :error , :already_created } end","ref":"aggregates.html#command-functions","title":"Aggregates - Command functions","type":"extras"},{"doc":"The state of an aggregate can only be mutated by applying a domain event to its state. This is achieved by an apply/2 function that receives the state and the domain event. It returns the modified state. Pattern matching is used to invoke the respective apply/2 function for an event. These functions must never fail as they are used when rebuilding the aggregate state from its history of domain events. You cannot reject the event once it has occurred. defmodule ExampleAggregate do def apply ( % ExampleAggregate { } , % Created { } = event ) do % Created { uuid : uuid , name : name } = event % ExampleAggregate { uuid : uuid , name : name } end end","ref":"aggregates.html#state-mutators","title":"Aggregates - State mutators","type":"extras"},{"doc":"You can define your aggregate with public API functions using the language of your domain. In this bank account example, the public function to open a new account is open_account/3 : defmodule BankAccount do defstruct [ :account_number , :balance ] # Public API def open_account ( % BankAccount { account_number : nil } , account_number , initial_balance ) when initial_balance &gt; 0 do event = % BankAccountOpened { account_number : account_number , initial_balance : initial_balance } { :ok , event } end def open_account ( % BankAccount { } , _account_number , initial_balance ) when initial_balance &lt;= 0 do { :error , :initial_balance_must_be_above_zero } end def open_account ( % BankAccount { } , _account_number , _initial_balance ) do { :error , :account_already_opened } end # State mutators def apply ( % BankAccount { } = account , % BankAccountOpened { } = event ) do % BankAccountOpened { account_number : account_number , initial_balance : initial_balance } = event % BankAccount { account | account_number : account_number , balance : initial_balance } end end With this approach you must dispatch the command to a command handler: defmodule Commanded.ExampleDomain.OpenAccountHandler do alias BankAccount alias Commands.OpenAccount @behaviour Commanded.Commands.Handler def handle ( % BankAccount { } = aggregate , % OpenAccount { } = command ) do % OpenAccount { account_number : account_number , initial_balance : initial_balance } = command BankAccount . open_account ( aggregate , account_number , initial_balance ) end end An alternative approach is to expose one or more public command functions, execute/2 , and use pattern matching on the command argument. With this approach you can route your commands directly to the aggregate. In this example the execute/2 function pattern matches on the OpenAccount command module: defmodule BankAccount do defstruct [ :account_number , :balance ] # Public API def execute ( % BankAccount { account_number : nil } , % OpenAccount { initial_balance : initial_balance } = command ) when initial_balance &gt; 0 do % OpenAccount { account_number : account_number } = command % BankAccountOpened { account_number : account_number , initial_balance : initial_balance } end def execute ( % BankAccount { } , % OpenAccount { initial_balance : initial_balance } ) when initial_balance &lt;= 0 do { :error , :initial_balance_must_be_above_zero } end def execute ( % BankAccount { } , % OpenAccount { } ) do { :error , :account_already_opened } end # State mutators def apply ( % BankAccount { } = account , % BankAccountOpened { } = event ) do % BankAccountOpened { account_number : account_number , initial_balance : initial_balance } = event % BankAccount { account | account_number : account_number , balance : initial_balance } end end","ref":"aggregates.html#example-aggregate","title":"Aggregates - Example aggregate","type":"extras"},{"doc":"Sometimes you need to create multiple events from a single command. You can use Commanded.Aggregate.Multi to help track the events and update the aggregate state. This can be useful when you want to emit multiple events that depend upon the aggregate state being updated. Any errors encountered will be returned to the caller, the modified aggregate state and any pending events are discarded.","ref":"aggregates.html#using-commanded-aggregate-multi-to-return-multiple-events","title":"Aggregates - Using Commanded.Aggregate.Multi to return multiple events","type":"extras"},{"doc":"In the example below, money is withdrawn from the bank account and the updated balance is used to check whether the account is overdrawn. defmodule BankAccount do defstruct [ account_number : nil , balance : 0 , state : nil , ] alias Commanded.Aggregate.Multi # Public API def execute ( % BankAccount { state : :active } = account , % WithdrawMoney { amount : amount } ) when is_number ( amount ) and amount &gt; 0 do account |&gt; Multi . new ( ) |&gt; Multi . execute ( &amp; withdraw_money ( &amp;1 , amount ) ) |&gt; Multi . execute ( &amp; check_balance / 1 ) end # State mutators def apply ( % BankAccount { } = state , % MoneyWithdrawn { balance : balance } ) , do : % BankAccount { state | balance : balance } def apply ( % BankAccount { } = state , % AccountOverdrawn { } ) , do : % BankAccount { state | state : :overdrawn } # Private helpers defp withdraw_money ( % BankAccount { } = account , amount ) do % BankAccount { account_number : account_number , balance : balance } = account event = % MoneyWithdrawn { account_number : account_number , amount : amount , balance : balance - amount } { :ok , event } end defp check_balance ( % BankAccount { balance : balance } = account ) when balance &lt; 0 do % BankAccount { account_number : account_number } = account event = % AccountOverdrawn { account_number : account_number , balance : balance } { :ok , event } end defp check_balance ( % BankAccount { } ) , do : :ok end","ref":"aggregates.html#example","title":"Aggregates - Example","type":"extras"},{"doc":"A snapshot represents the aggregate state when all events to that point in time have been replayed. You can optionally configure state snapshotting for individual aggregates in your app configuration. Instead of loading every event for an aggregate when rebuilding its state, only the snapshot and any events appended since its creation are read. By default snapshotting is disabled for all aggregates. As an example, assume a snapshot was taken after persisting an event for the aggregate at version 100. When the aggregate process is restarted we load and deserialize the snapshot data as the aggregate's initial state. Then we fetch and replay the aggregate's events after version 100. This is a performance optimisation for aggregates that have a long lifetime or raise a large number of events. It limits the worst case scenario when rebuilding the aggregate state: it will need to read the snapshot and at most this many events from storage. Use the following options to configure snapshots for an aggregate: snapshot_every - snapshot aggregate state every so many events. Use nil to disable snapshotting, or exclude the configuration entirely. snapshot_version - a non-negative integer indicating the version of the aggregate state snapshot. Incrementing this version forces any earlier recorded snapshots to be ignored when rebuilding aggregate state. Example In config/config.exs enable snapshots for ExampleAggregate after every ten events: config :my_app , MyApp.Application , snapshotting : %{ MyApp.ExampleAggregate =&gt; [ snapshot_every : 10 , snapshot_version : 1 ] } Snapshot serialization Aggregate state will be serialized using the configured event store serializer, by default this stores the data as JSON. Remember to derive the Jason.Encoder protocol for the aggregate state to ensure JSON serialization is supported, as shown below. defmodule ExampleAggregate do @derive Jason.Encoder defstruct [ :name , :datetime ] end You can use the Commanded.Serialization.JsonDecoder protocol to decode the parsed JSON data into the expected types: defimpl Commanded.Serialization.JsonDecoder , for : ExampleAggregate do @doc &quot;&quot;&quot; Parse the datetime included in the aggregate state &quot;&quot;&quot; def decode ( % ExampleAggregate { } = state ) do % ExampleAggregate { datetime : datetime } = state { :ok , dt , _ } = DateTime . from_iso8601 ( datetime ) % ExampleAggregate { state | datetime : dt } end end Note: The default JSON encoding of a DateTime struct uses the to_iso8601/1 function which is why we must decode it using the from_iso8601/1 function. Rebuilding an aggregate snapshot Whenever you change the structure of an aggregate's state you must increment the snapshot_version number. The aggregate state will be rebuilt from its events, ignoring any existing snapshots. They will be overwritten when the next snapshot is taken.","ref":"aggregates.html#aggregate-state-snapshots","title":"Aggregates - Aggregate state snapshots","type":"extras"},{"doc":"You need to create a module per command and define the fields using defstruct : defmodule OpenAccount do @enforce_keys [ :account_number ] defstruct [ :account_number , :initial_balance ] end A command must contain a field to uniquely identify the aggregate instance (e.g. account_number ). Use @enforce_keys to force the identity field to be specified when creating the command struct. Since commands are just plain Elixir structs you can use a library such as typed_struct for defining structs, fields with their types, and enforcing mandatory keys without writing too much boilerplate code. Commanded logs commands during execution when using debug log level. To prevent sensitive data from being exposed in logs you can use the built-in Elixir Inspect module to exclude fields. defmodule RegisterUser do @derive { Inspect , only : [ :id , :name ] } defstruct [ :id , :name , :password ] end","ref":"commands.html","title":"Commands","type":"extras"},{"doc":"A router module is used to route commands to their registered command handler and aggregate module. You create a router module using Commanded.Commands.Router and register each command with its associated handler: defmodule BankRouter do use Commanded.Commands.Router dispatch OpenAccount , to : OpenAccountHandler , aggregate : BankAccount , identity : :account_number dispatch DepositMoney , to : DepositMoneyHandler , aggregate : BankAccount , identity : :account_number end This can be more succinctly configured by excluding command handlers and dispatching directly to the aggregate , using multi-command registration , and with the identify helper macro: defmodule BankRouter do use Commanded.Commands.Router identify BankAccount , by : :account_number dispatch [ OpenAccount , DepositMoney ] , to : BankAccount end A router must be registered with its associated application using the router macro, as shown below: defmodule BankApp do use Commanded.Application , otp_app : :bank_app router BankRouter end","ref":"commands.html#command-dispatch-and-routing","title":"Commands - Command dispatch and routing","type":"extras"},{"doc":"A command handler receives the aggregate and the command being executed. It allows you to validate, authorize, and/or enrich the command with additional data before executing the appropriate aggregate module function. The command handler must implement the Commanded.Commands.Handler behaviour consisting of a single handle/2 function. It receives the aggregate state and the command to be handled. It must return the raised domain events from the aggregate. It may return an {:error, reason} tuple on failure. The handle/2 function is executed in the same process as the aggregate itself and is therefore within the consistency boundary of the aggregate. In effect, there is no essential difference between using the handle/2 function in a command handler module vs. the execute/2 function in the aggregate module aside from organization of your source code. defmodule OpenAccountHandler do @behaviour Commanded.Commands.Handler def handle ( % BankAccount { } = aggregate , % OpenAccount { } = command ) do % OpenAccount { account_number : account_number , initial_balance : initial_balance } = command BankAccount . open_account ( aggregate , account_number , initial_balance ) end end Command handlers execute in the context of the dispatch call, as such they are limited to the timeout period specified. The default timeout is five seconds, the same as a GenServer call. You can increase the timeout value for individual commands as required - see the section on Timeouts below. Dispatch directly to aggregate It is also possible to route a command directly to an aggregate, without requiring an intermediate command handler. defmodule BankRouter do use Commanded.Commands.Router # will route to `BankAccount.execute/2` dispatch OpenAccount , to : BankAccount , identity : :account_number end By default, the aggregate module's execute/2 function will be called with the aggregate's state and the command to execute. Using this approach, you will create an execute/2 clause that pattern-matches on each command that the aggregate should handle. Alternatively, you may specify the name of a function (also receiving both the aggregate state and the command) on your aggregate module to which the command will be dispatched: defmodule BankRouter do use Commanded.Commands.Router # Will route to `BankAccount.open_account/2` dispatch OpenAccount , to : BankAccount , function : :open_account , identity : :account_number end Dispatching commands You dispatch a command to its registered aggregate using the application: :ok = BankApp . dispatch ( % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } ) Optionally, you can dispatch a command using the router by providing the application as an option: command = % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } :ok = BankRouter . dispatch ( command , application : BankApp ) Define aggregate identity You can define the identity field for an aggregate once using the identify macro. The configured identity will be used for all commands registered to the aggregate, unless overridden by a command registration. Example defmodule BankRouter do use Commanded.Commands.Router identify BankAccount , by : :account_number dispatch OpenAccount , to : BankAccount end The above configuration requires that all commands for the BankAccount aggregate must contain an account_number field used to identity a unique instance. Identity prefix An optional identity prefix can be used to distinguish between different aggregates that would otherwise share the same identity. As an example you might have a User and a UserPreferences aggregate that you wish to share the same identity. In this scenario you should specify a prefix for each aggregate (e.g. &quot;user-&quot; and &quot;user-preference-&quot;). defmodule BankRouter do use Commanded.Commands.Router identify BankAccount , by : :account_number , prefix : &quot;bank-account-&quot; dispatch OpenAccount , to : BankAccount end The prefix is used as the stream identity when appending, and reading, the aggregate's events (e.g. &lt;prefix&gt;&lt;instance_identity&gt; ). Note you must not change the stream prefix once you have events persisted in your event store, otherwise the aggregate's events cannot be read from the event store and its state cannot be rebuilt since the stream name will be different. Custom aggregate identity Any module that implements the String.Chars protocol can be used for an aggregate's identity. By default this includes the following Elixir built-in types: strings, integers, floats, atoms, and lists. You can define your own custom identity modules and implement the String.Chars protocol for them: defmodule AccountNumber do defstruct [ :branch , :account_number ] defimpl String.Chars do def to_string ( % AccountNumber { branch : branch , account_number : account_number } ) , do : branch &lt;&gt; &quot;:&quot; &lt;&gt; account_number end end The custom identity will be converted to a string during command dispatch. This is used as the aggregate's identity and determines the stream to append its events in the event store. open_account = % OpenAccount { account_number : % AccountNumber { branch : &quot;B1&quot; , account_number : &quot;ACC123&quot; } , initial_balance : 1_000 } :ok = BankApp . dispatch ( open_account ) Timeouts A command handler has a default timeout of 5 seconds. The same default as a GenServer.call/3 process call. It must handle the command in this period, otherwise the call fails and the caller process exits. You can configure a different timeout value during command registration by providing a timeout option, defined in milliseconds: defmodule BankRouter do use Commanded.Commands.Router # Configure a timeout of 1 second for the open account command handler dispatch OpenAccount , to : OpenAccountHandler , aggregate : BankAccount , identity : :account_number , timeout : 1_000 end You can override the timeout value during command dispatch. This example is dispatching the open account command with a timeout of 2 seconds: open_account = % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } :ok = BankApp . dispatch ( open_account , timeout : 2_000 ) Multi-command registration Command routers support multi command registration so you can group related command handlers into the same module: defmodule BankRouter do use Commanded.Commands.Router dispatch [ OpenAccount , CloseAccount ] , to : BankAccountHandler , aggregate : BankAccount , identity : :account_number end Command dispatch consistency guarantee You can choose the consistency guarantee when dispatching a command. Strong consistency offers up-to-date data but at the cost of high latency. Eventual consistency offers low latency but read model queries may reply with stale data since they may not have processed the persisted events. In Commanded, the available options during command dispatch are: :eventual (default) - don't block command dispatch and don't wait for any event handlers, regardless of their own consistency configuration. :ok = BankApp . dispatch ( command ) :ok = BankApp . dispatch ( command , consistency : :eventual ) :strong - block command dispatch until all strongly consistent event handlers and process managers have successfully processed all events created by the command. :ok = BankApp . dispatch ( command , consistency : :strong ) Dispatching a command using :strong consistency but without any strongly consistent event handlers configured will have no effect. Provide an explicit list of event handler and process manager modules (or their configured names), containing only those handlers you'd like to wait for. No other handlers will be awaited on, regardless of their own configured consistency setting. :ok = BankApp . dispatch ( command , consistency : [ ExampleHandler , AnotherHandler ] ) :ok = BankApp . dispatch ( command , consistency : [ &quot;ExampleHandler&quot; , &quot;AnotherHandler&quot; ] ) Note you cannot opt-in to strong consistency for a handler that has been configured as eventually consistent. Which consistency guarantee should I use? When dispatching a command using consistency: :strong the dispatch will block until all of the strongly consistent event handlers and process managers have handled all events created by the command. This guarantees that when you receive the :ok response from dispatch, your strongly consistent read models will have been updated and can safely be queried. Strong consistency helps to alleviate problems and workarounds you would otherwise encounter when dealing with eventual consistency in your own application. Use :strong consistency when you want to query a read model immediately after dispatching a command. You must also configure the event handler to use :strong consistency. Using :eventual consistency, or omitting the consistency option, will cause the command dispatch to immediately return without waiting for any event handlers or process managers. The handlers run independently, and asynchronously, in the background, therefore you will need to deal with potentially stale read model data. Configure default consistency You may override the default consistency ( :eventual ) by setting default_consistency in your environment config (e.g. config/config.exs ): config :commanded , default_consistency : :strong This will effect command dispatch, event handlers, and process managers where a consistency is not explicitly defined. Consistency failures By opting-in to strong consistency you may encounter an additional error reply from command dispatch: case BankApp . dispatch ( command , consistency : :strong ) do :ok -&gt; # ... all ok { :error , :consistency_timeout } -&gt; # command ok, handlers have not yet executed end Receiving an {:error, :consistency_timeout} error indicates the command successfully dispatched, but some or all of the strongly consistent event handlers have not yet executed. The default timeout is configured as five seconds; this determines how long the dispatch will block waiting for the handlers. You can override the default value in your environment config file (e.g. config/config.exs ): config :commanded , dispatch_consistency_timeout : 10_000 # ten seconds Dispatch returning execution result You can choose to include the execution result as part of the dispatch result by setting include_execution_result true: { :ok , % Commanded.Commands.ExecutionResult { aggregate_uuid : aggregate_uuid , aggregate_version : aggregate_version , events : events , metadata : metadata } } = BankApp . dispatch ( command , include_execution_result : true ) This is useful if you need to get information from the events produced by the aggregate. Dispatch returning aggregate version You can optionally choose to include the aggregate's version as part of the dispatch result by setting the include_aggregate_version option to true: { :ok , aggregate_version } = BankApp . dispatch ( command , include_aggregate_version : true ) This is useful when you need to wait for an event handler, such as a read model projection, to be up-to-date before continuing execution or querying its data. Causation and correlation ids To assist with monitoring and debugging your deployed application it is useful to track the causation and correlation ids for your commands and events. causation_id - the UUID of the command causing an event, or the event causing a command dispatch. correlation_id - a UUID used to correlate related commands/events. You can set causation and correlation ids when dispatching a command: :ok = BankApp . dispatch ( command , causation_id : UUID . uuid4 ( ) , correlation_id : UUID . uuid4 ( ) ) When dispatching a command in an event handler, you should copy these values from the metadata (second) argument associated with the event you are handling: defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : &quot;ExampleHandler&quot; def handle ( % AnEvent { .. } , %{ event_id : causation_id , correlation_id : correlation_id } ) do command = % ExampleCommand { .. } BankApp . dispatch ( command , causation_id : causation_id , correlation_id : correlation_id , ) end end Commands dispatched by a process manager will be automatically assigned the appropriate causation and correlation ids from the source domain event. You can use Commanded audit middleware to record every dispatched command. This allows you to follow the chain of commands and events by using the causation id. The correlation id can be used to find all related commands and events. Event metadata It's helpful for debugging to have additional metadata associated with events issued by a command. You can set it when dispatching a command: :ok = BankApp . dispatch ( command , metadata : %{ &quot;issuer_id&quot; =&gt; issuer_id , &quot;user_id&quot; =&gt; &quot;user@example.com&quot; } ) Note, due to metadata serialization you should expect that only: strings, numbers, and boolean values are preserved; any other value will be converted to a string. You should always use string keys in your metadata map; atom keys will be converted to strings. Aggregate lifespan By default an aggregate instance process will run indefinitely once started. You can control this by implementing the Commanded.Aggregates.AggregateLifespan behaviour in a module. Define a module that implements the Commanded.Aggregates.AggregateLifespan behaviour: defmodule BankAccountLifespan do @behaviour Commanded.Aggregates.AggregateLifespan def after_event ( % MoneyDeposited { } ) , do : :timer . hours ( 1 ) def after_event ( % BankAccountClosed { } ) , do : :stop def after_event ( _event ) , do : :infinity def after_command ( % CloseAccount { } ) , do : :stop def after_command ( _command ) , do : :infinity def after_error ( :invalid_initial_balance ) , do : :timer . minutes ( 5 ) def after_error ( _error ) , do : :stop end Then specify the module as the lifespan option when registering the command in the router. defmodule BankRouter do use Commanded.Commands.Router dispatch [ OpenAccount , CloseAccount ] , to : BankAccount , lifespan : BankAccountLifespan , identity : :account_number end The inactivity timeout is specified in milliseconds, after which time the aggregate process will be stopped if no other messages are received by it. Return :stop or {:stop, reason} to immediately shutdown the aggregate process. Return :infinity to prevent the aggregate instance from shutting down. You can also return :hibernate and the process is hibernated, it will continue its loop once a message is in its message queue. Hibernating an aggregate causes garbage collection and minimises the memory used by the process. Hibernating should not be used aggressively as too much time could be spent garbage collecting.","ref":"commands.html#command-handlers","title":"Commands - Command handlers","type":"extras"},{"doc":"Allows a command router to define middleware modules that are executed before and after success or failure of each command dispatch. This provides an extension point to add in command validation, authorization, logging, and other cross-cutting behaviour that you want to be called for every command the router dispatches. defmodule BankingRouter do use Commanded.Commands.Router middleware CommandLogger middleware MyCommandValidator middleware AuthorizeCommand identify BankAccount , by : :account_number dispatch OpenAccount , to : OpenAccountHandler , aggregate : BankAccount dispatch DepositMoney , to : DepositMoneyHandler , aggregate : BankAccount end The middleware modules are executed in the order they’ve been defined. They will receive a Commanded.Middleware.Pipeline struct containing the command being dispatched, in the :command field. Example middleware Implement the Commanded.Middleware behaviour in your module and define the before_dispatch , after_dispatch , and after_failure callback functions. defmodule NoOpMiddleware do @behaviour Commanded.Middleware alias Commanded.Middleware.Pipeline import Pipeline def before_dispatch ( % Pipeline { } = pipeline ) do pipeline end def after_dispatch ( % Pipeline { } = pipeline ) do pipeline end def after_failure ( % Pipeline { } = pipeline ) do pipeline end end Commanded provides a Commanded.Middleware.Logger middleware for logging the name of each dispatched command and its execution duration.","ref":"commands.html#middleware","title":"Commands - Middleware","type":"extras"},{"doc":"You can use the Commanded.Commands.CompositeRouter macro to define a router comprised of other router modules. This approach is useful if you prefer to construct a router per context and then combine them together to form a top level application router. By using Commanded.Commands.CompositeRouter in your own module you can include other routers via the router macro: defmodule ApplicationRouter do use Commanded.Commands.CompositeRouter router BankAccountRouter router MoneyTransferRouter end defmodule BankApp do use Commanded.Application , otp_app : :bank_app router ApplicationRouter end Command dispatch works the same as any other router: :ok = BankApp . dispatch ( % OpenAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } )","ref":"commands.html#composite-command-routers","title":"Commands - Composite command routers","type":"extras"},{"doc":"","ref":"events.html","title":"Events","type":"extras"},{"doc":"Domain events indicate that something of importance has occurred, within the context of an aggregate. They should be named in the past tense: account registered; funds transferred; fraudulent activity detected etc. Create a module per domain event and define the fields with defstruct . An event should contain a field to uniquely identify the aggregate instance (e.g. account_number ). Remember to derive the Jason.Encoder protocol for the event struct to ensure JSON serialization is supported, as shown below. defmodule BankAccountOpened do @derive Jason.Encoder defstruct [ :account_number , :initial_balance ] end Note, due to event serialization you should expect that only: strings, numbers and boolean values defined in an event are preserved; any other value will be converted to a string. You can control this behaviour as described in the Serialization guide .","ref":"events.html#domain-events","title":"Events - Domain events","type":"extras"},{"doc":"Event handlers allow you to execute code that reacts to domain events: to build read model projections; dispatch commands to other aggregates; and to interact with third-party systems such as sending emails. Commanded guarantees only one instance of an event handler will run, regardless of how many nodes are running (even when not using distributed Erlang). This is enforced by the event store subscription (PostgreSQL advisory locks in Elixir Event Store). Use the Commanded.Event.Handler macro within your event handler module to implement the defined behaviour. This consists of a single handle/2 function that receives each published domain event and its metadata, including the event's unique event number. It should return :ok on success or {:error, :reason} on failure. You can return {:error, :already_seen_event} to skip events that have already been handled, due to the at-least-once event delivery of the supported event stores. Use pattern matching to match on each type of event you are interested in. A catch-all handle/2 function is included, so all other events will be ignored by default. defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : &quot;ExampleHandler&quot; def handle ( % AnEvent { .. } , _metadata ) do # ... process the event :ok end end The name given to the event handler must be unique and remain unchanged between releases. It is used when subscribing to the event store to track which events the handler has seen during restarts. { :ok , _handler } = ExampleHandler . start_link ( ) Subscription options You can choose to start the event handler's event store subscription from the :origin , :current position, or an exact event number using the start_from option. The default is to use the origin so your handler will receive all events. defmodule ExampleHandler do # Define `start_from` as one of :origin, :current, or an explicit event number (e.g. 1234) use Commanded.Event.Handler , application : ExampleApp , name : &quot;ExampleHandler&quot; , start_from : :origin end You can optionally override :start_from by passing it as param: { :ok , _handler } = ExampleHandler . start_link ( start_from : :current ) Use the :current position when you don't want newly created event handlers to go through all previous events. An example would be adding an event handler to send transactional emails to an already deployed system containing many historical events. You should start your event handlers using an OTP Supervisor to ensure they are restarted on error. See the Supervision guide for more details. Subscribing to an individual stream By default event handlers will subscribe to all events appended to any stream. Provide a subscribe_to option to subscribe to a single stream. defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : __MODULE__ , subscribe_to : &quot;stream1234&quot; end This will ensure the handler only receives events appended to that stream. Event handler callbacks Commanded.Event.Handler.init/0 - (optional) initialisation callback function called when the handler starts. Commanded.Event.Handler.init/1 - (optional) used to configure the handler before it starts. Commanded.Event.Handler.error/3 - (optional) called when an event handle/2 callback returns an error. Metadata The handle/2 function in your handler receives the domain event and a map of metadata associated with that event. You can provide the metadata key/value pairs when dispatching a command: :ok = ExampleApp . dispatch ( command , metadata : %{ &quot;issuer_id&quot; =&gt; issuer_id , &quot;user_id&quot; =&gt; &quot;user@example.com&quot; } ) In addition to the metadata key/values you provide, the following system values will be included in the metadata passed to an event handler: application - the Commanded.Application associated with the event handler. handler_name - the name of the event handler. state - optional event handler state. event_id - a globally unique UUID to identify the event. event_number - a globally unique, monotonically incrementing and gapless integer used to order the event amongst all events. stream_id - the stream identity for the event. stream_version - the version of the stream for the event. causation_id - an optional UUID identifier used to identify which command caused the event. correlation_id - an optional UUID identifier used to correlate related commands/events. created_at - the datetime, in UTC, indicating when the event was created. These key/value metadata pairs will use atom keys to differentiate them from the user provided metadata: defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : &quot;ExampleHandler&quot; def handle ( event , metadata ) do IO . inspect ( metadata ) # %{ # :causation_id =&gt; &quot;db1ebd30-7d3c-40f7-87cd-12cd9966df32&quot;, # :correlation_id =&gt; &quot;1599630b-9c38-433c-9548-0dd793108ba0&quot;, # :created_at =&gt; #DateTime&lt;2017-10-30 11:19:56.178901Z&gt;, # :event_id =&gt; &quot;5e4a0f38-385b-4d57-823b-a1bcf705b7bb&quot;, # :event_number =&gt; 12345, # :stream_id =&gt; &quot;e42a588d-2cda-4314-a471-5d008cce01fc&quot;, # :stream_version =&gt; 1, # &quot;issuer_id&quot; =&gt; &quot;0768d69a-d2b7-48f4-d0e9-083a97f7ebe0&quot;, # &quot;user_id&quot; =&gt; &quot;user@example.com&quot; # } :ok end end Consistency guarantee You can specify an event handler's consistency guarantee using the consistency option: defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : &quot;ExampleHandler&quot; , consistency : :eventual The available options are :eventual (default) and :strong : Strong consistency offers up-to-date data but at the cost of high latency. Eventual consistency offers low latency but read model queries may reply with stale data since they may not have processed the persisted events. You request the consistency guarantee, either :strong or :eventual , when dispatching a command. Strong consistency will block the command dispatch and wait for all strongly consistent event handlers to successfully process all events created by the command. Whereas eventual consistency will immediately return after command dispatch, without waiting for any event handlers, even those configured for strong consistency. How does it work? An event handler is a GenServer process that subscribes to the configured event store. For each event persisted to the store the handle/2 callback is called, passing the domain event and its metadata.","ref":"events.html#event-handlers","title":"Events - Event handlers","type":"extras"},{"doc":"Commanded supports upcasting of events at runtime using the Commanded.Event.Upcaster protocol. By implementing the upcaster protocol you can transform an event before it is used by a consumer. This might be an aggregate, an event handler, or a process manager. Because the upcaster changes the event at runtime, handlers only need to support the latest version. You can also use upcasting to change the type of event. Examples Change the shape of an event by renaming a field: defimpl Commanded.Event.Upcaster , for : AnEvent do def upcast ( % AnEvent { } = event , _metadata ) do % AnEvent { name : name } = event % AnEvent { event | first_name : name } end end Change the type of event by replacing a historical event with a new event: defimpl Commanded.Event.Upcaster , for : HistoricalEvent do def upcast ( % HistoricalEvent { } = event , _metadata ) do % HistoricalEvent { id : id , name : name } = event % NewEvent { id : id , name : name } end end","ref":"events.html#upcasting-events","title":"Events - Upcasting events","type":"extras"},{"doc":"An event handler can be reset (using a mix task), it will restart the event store subscription from the configured start_from . This allow an individual handler to be restart while the app is still running. You can define a before_reset/1 function that will be called before resetting the event handler. defmodule ExampleHandler do use Commanded.Event.Handler , application : ExampleApp , name : __MODULE__ require Logger alias Commanded.Event.FailureContext def before_reset do # Do something :ok end end","ref":"events.html#reset-an-eventhandler","title":"Events - Reset an EventHandler","type":"extras"},{"doc":"A process manager is responsible for coordinating one or more aggregates. It handles events and dispatches commands in response. You can think of a process manager as the opposite of an aggregate: aggregates handle commands and create events; process managers handle events and create commands. Process managers have state that can be used to track which aggregates are being orchestrated. Use the Commanded.ProcessManagers.ProcessManager macro in your process manager module and implement the callback functions defined in the behaviour: interested?/1 , handle/2 , apply/2 , and error/3 .","ref":"process-managers.html","title":"Process managers","type":"extras"},{"doc":"The interested?/1 function is used to indicate which events the process manager handles. The response is used to route the event to an existing instance or start a new process instance: {:start, process_uuid} - create a new instance of the process manager. {:start!, process_uuid} - create a new instance of the process manager (strict). {:continue, process_uuid} - continue execution of an existing process manager. {:continue!, process_uuid} - continue execution of an existing process manager (strict). {:stop, process_uuid} - stop an existing process manager, shutdown its process, and delete its persisted state. false - ignore the event. You can return a list of process identifiers when a single domain event is to be handled by multiple process instances. Strict process routing Using strict routing, with :start! or :continue! , enforces the following validation checks: {:start!, process_uuid} - validate process does not already exist. {:continue!, process_uuid} - validate process already exists. If the check fails an error will be passed to the error/3 callback function: {:error, {:start!, :process_already_started}} {:error, {:continue!, :process_not_started}} The error/3 function can choose to :stop the process or :skip the problematic event.","ref":"process-managers.html#interested-1","title":"Process managers - interested?/1","type":"extras"},{"doc":"A handle/2 function can be defined for each :start and :continue tagged event previously specified. It receives the process manager's state and the event to be handled. It must return the commands to be dispatched. This may be none, a single command, or many commands. The handle/2 function can be omitted if you do not need to dispatch a command and are only mutating the process manager's state.","ref":"process-managers.html#handle-2","title":"Process managers - handle/2","type":"extras"},{"doc":"The apply/2 function is used to mutate the process manager's state. It receives the current state and the domain event, and must return the modified state. This callback function is optional, the default behaviour is to retain the process manager's current state.","ref":"process-managers.html#apply-2","title":"Process managers - apply/2","type":"extras"},{"doc":"You can define an c:error/3 callback function to handle any errors or exceptions during event handling or returned by commands dispatched from your process manager. The function is passed the error (e.g. {:error, :failure} ), the failed event or command, and a failure context. See Commanded.ProcessManagers.FailureContext for details. Use pattern matching on the error and/or failed event/command to explicitly handle certain errors, events, or commands. You can choose to retry, skip, ignore, or stop the process manager after a command dispatch error. The default behaviour, if you don't provide an c:error/3 callback, is to stop the process manager using the exact error reason returned from the event handler function or command dispatch. The error/3 callback function must return one of the following responses depending upon the severity of error and how you choose to handle it: {:retry, context} - retry the failed command, provide a context map containing any state passed to subsequent failures. This could be used to count the number of retries, failing after too many attempts. {:retry, delay, context} - retry the failed command, after sleeping for the requested delay (in milliseconds). Context is a map as described in {:retry, context} above. {:stop, reason} - stop the process manager with the given reason. For event handling failures, when failure source is an event, you can also return: :skip - to skip the problematic event. No commands will be dispatched. For command dispatch failures, when failure source is a command, you can also return: {:continue, commands, context} - continue dispatching the given commands. This allows you to retry the failed command, modify it and retry, drop it, or drop all pending commands by passing an empty list [] . {:skip, :discard_pending} - discard the failed command and any pending commands. {:skip, :continue_pending} - skip the failed command, but continue dispatching any pending commands.","ref":"process-managers.html#error-3","title":"Process managers - error/3","type":"extras"},{"doc":"Supervise your process managers to ensure they are restarted on error. defmodule Bank.Payments.Supervisor do use Supervisor def start_link ( _arg ) do Supervisor . start_link ( __MODULE__ , [ ] , name : __MODULE__ ) end def init ( _arg ) do Supervisor . init ( [ Bank.Payments.TransferMoneyProcessManager ] , strategy : :one_for_one ) end end Supervision caveats The default error handling strategy is to stop the process manager. When supervised, the process will be restarted and will attempt to handle the same event again, which will likely result in the same error. This could lead to too many restarts of the supervisor, which may eventually cause the application to stop, depending upon your supervision tree and its strategy. To prevent this you can choose to define the default error handling strategy as :skip to skip over any problematic events. defmodule ExampleProcessManager do use Commanded.ProcessManagers.ProcessManager , application : ExampleApp , name : __MODULE__ require Logger # By default skip any problematic events def error ( error , _command_or_event , _failure_context ) do Logger . error ( fn -&gt; &quot; \#{ __MODULE__ } encountered an error: &quot; &lt;&gt; inspect ( error ) end ) :skip end end Alternatively you can define the restart strategy of your process manager as :temporary to prevent it from being restarted on termination. This approach will require manual intervention to fix the stopped process manager, but ensures that it won't miss any events nor crash the application. Error handling example Define an error/3 callback function to determine how to handle errors during event handling and command dispatch. defmodule ExampleProcessManager do use Commanded.ProcessManagers.ProcessManager , application : ExampleApp , name : &quot;ExampleProcessManager&quot; # Stop process manager after three failures def error ( { :error , _failure } , _failed_message , %{ context : %{ failures : failures } } ) when failures &gt;= 2 do { :stop , :too_many_failures } end # Retry command, record failure count in context map def error ( { :error , _failure } , _failed_message , %{ context : context } ) do context = Map . update ( context , :failures , 1 , fn failures -&gt; failures + 1 end ) { :retry , context } end end The default behaviour if you don't provide an error/3 callback is to stop the process manager using the same error reason returned from the failed command dispatch.","ref":"process-managers.html#supervision","title":"Process managers - Supervision","type":"extras"},{"doc":"defmodule TransferMoneyProcessManager do use Commanded.ProcessManagers.ProcessManager , application : ExampleApp , name : &quot;TransferMoneyProcessManager&quot; @derive Jason.Encoder defstruct [ :transfer_uuid , :debit_account , :credit_account , :amount , :status ] # Process routing def interested? ( % MoneyTransferRequested { transfer_uuid : transfer_uuid } ) , do : { :start , transfer_uuid } def interested? ( % MoneyWithdrawn { transfer_uuid : transfer_uuid } ) , do : { :continue , transfer_uuid } def interested? ( % MoneyDeposited { transfer_uuid : transfer_uuid } ) , do : { :stop , transfer_uuid } def interested? ( _event ) , do : false # Command dispatch def handle ( % TransferMoneyProcessManager { } , % MoneyTransferRequested { } = event ) do % MoneyTransferRequested { transfer_uuid : transfer_uuid , debit_account : debit_account , amount : amount } = event % WithdrawMoney { account_number : debit_account , transfer_uuid : transfer_uuid , amount : amount } end def handle ( % TransferMoneyProcessManager { } = pm , % MoneyWithdrawn { } ) do % TransferMoneyProcessManager { transfer_uuid : transfer_uuid , credit_account : credit_account , amount : amount } = pm % DepositMoney { account_number : credit_account , transfer_uuid : transfer_uuid , amount : amount } end # State mutators def apply ( % TransferMoneyProcessManager { } = transfer , % MoneyTransferRequested { } = event ) do % MoneyTransferRequested { transfer_uuid : transfer_uuid , debit_account : debit_account , credit_account : credit_account , amount : amount } = event % TransferMoneyProcessManager { transfer | transfer_uuid : transfer_uuid , debit_account : debit_account , credit_account : credit_account , amount : amount , status : :withdraw_money_from_debit_account } end def apply ( % TransferMoneyProcessManager { } = transfer , % MoneyWithdrawn { } ) do % TransferMoneyProcessManager { transfer | status : :deposit_money_in_credit_account } end end The name given to the process manager must be unique. This is used when subscribing to events from the event store to track the last seen event and ensure they are only received once. { :ok , _pid } = TransferMoneyProcessManager . start_link ( start_from : :current ) You can choose to start the process router's event store subscription from the :origin , :current position or an exact event number using the start_from option. The default is to use :origin so it will receive all events. You typically use :current when adding a new process manager to an already deployed system containing historical events. Process manager instance state is persisted to storage after each handled event. This allows the process manager to resume should the host process terminate.","ref":"process-managers.html#example-process-manager","title":"Process managers - Example process manager","type":"extras"},{"doc":"consistency - defined as one of either :strong or :eventual (default) for event handling. event_timeout - a timeout for event handling to ensure that events are processed in a timely manner without getting stuck. idle_timeout - to reduce memory usage you can configure an idle timeout, in milliseconds, after which an inactive process instance will be shutdown. Refer to the Commanded.ProcessManagers.ProcessManager module docs for more details.","ref":"process-managers.html#configuration-options","title":"Process managers - Configuration options","type":"extras"},{"doc":"Use an OTP supervisor to host your Commanded application, process managers, event handlers, and read model projectors. defmodule Bank.Supervisor do use Supervisor def start_link ( arg ) do Supervisor . start_link ( __MODULE__ , arg , name : __MODULE__ ) end @impl true def init ( _arg ) do children = [ # Application BankApp , # Event handler AccountBalanceHandler , # Process manager TransferMoneyProcessManager , # Read model projector AccountsProjector , # Optionally, provide runtime configuration { WelcomeEmailHandler , start_from : :current } , ] Supervisor . init ( children , strategy : :one_for_one ) end end Your application should start the supervisor: defmodule Bank do use Application def start ( _type , _args ) do Bank.Supervisor . start_link ( ) end end","ref":"supervision.html","title":"Supervision","type":"extras"},{"doc":"","ref":"serialization.html","title":"Serialization","type":"extras"},{"doc":"JSON serialization can be used for event data &amp; metadata, and aggregate and process manager snapshots. To enable JSON serialization with the included Commanded.Serialization.JsonSerializer module add the jason library to your deps: def deps do [ { :jason , &quot;~&gt; 1.2&quot; } ] end Jason , a pure Elixir JSON library, is used for the actual serialization. You must derive the Jason.Encoder protocol for all structs you plan on encoding. defmodule ExampleEvent do @derive Jason.Encoder defstruct [ :name , :datetime ] end Jason provides an extension point if you need to manually encode your event by using the Jason.Encoder protocol: defimpl Jason.Encoder , for : Person do def encode ( %{ name : name , age : age } , opts ) do Jason.Encode . string ( &quot; \#{ name } ( \#{ age } )&quot; , options ) end end","ref":"serialization.html#default-json-serializer","title":"Serialization - Default JSON serializer","type":"extras"},{"doc":"JSON serialization configuration depends upon which event store you are using with Commanded. Configure the serializer in config/config.exs , or per environment, as follows. Postgres EventStore: config :my_app , MyApp.EventStore , serializer : Commanded.Serialization.JsonSerializer You can also use the EventStore.JsonSerializer ( bytea column type) and EventStore.JsonbSerializer ( jsonb column type) serializers which are included in the EventStore library. config :my_app , MyApp.EventStore , serializer : EventStore.JsonSerializer config :my_app , MyApp.EventStore , serializer : EventStore.JsonbSerializer Note that the two EventStore serializers do not implement the Commanded.Serialization.JsonSerializer decoding protocol. EventStoreDB: config :commanded_extreme_adapter , serializer : Commanded.Serialization.JsonSerializer In-memory event store: config :commanded , Commanded.EventStore.Adapters.InMemory , serializer : Commanded.Serialization.JsonSerializer","ref":"serialization.html#configuring-json-serialization","title":"Serialization - Configuring JSON serialization","type":"extras"},{"doc":"The Commanded.Serialization.JsonSerializer module provides an extension point to allow additional decoding of the deserialized data. This can be used for parsing data into valid types, such as datetime parsing from a string. The example event below has an implementation of the Commanded.Serialization.JsonDecoder protocol to parse the datetime into a DateTime struct. defmodule ExampleEvent do @derive Jason.Encoder defstruct [ :name , :datetime ] end defimpl Commanded.Serialization.JsonDecoder , for : ExampleEvent do @doc &quot;&quot;&quot; Parse the datetime included in the event. &quot;&quot;&quot; def decode ( % ExampleEvent { datetime : datetime } = event ) do { :ok , dt , _ } = DateTime . from_iso8601 ( datetime ) % ExampleEvent { event | datetime : dt } end end The above protocol is only supported by the Commanded.Serialization.JsonSerializer serializer.","ref":"serialization.html#decoding-event-structs","title":"Serialization - Decoding event structs","type":"extras"},{"doc":"You can implement your own serializer module to use an alternative serialization format if preferred. Configure your own serializer in config/config.exs for the event store you are using. PostgreSQL EventStore: config :my_app , MyApp.EventStore , serializer : MyApp.MessagePackSerializer EventStoreDB: config :my_app , MyApp , serializer : MyApp.MessagePackSerializer In-memory event store: config :my_app , MyApp , event_store : [ adapter : Commanded.EventStore.Adapters.InMemory , serializer : MyApp.MessagePackSerializer ] You should not change serialization format once your app has been deployed to production since Commanded will not be able to deserialize any existing events or snapshot data. In this scenario, to change serialization format you would need to also migrate your event store to the new format.","ref":"serialization.html#using-an-alternative-serialization-format","title":"Serialization - Using an alternative serialization format","type":"extras"},{"doc":"To implement custom serialization you could implement your own serializer which extends the behaviour of the existing Commanded.Serialization.JsonSerializer module. The example below shows how you might compress the serialized data before storage and decompress during deserialization. defmodule CompressedJsonSerializer do def serialize ( term ) do term |&gt; Commanded.Serialization.JsonSerializer . serialize ( ) |&gt; compress ( ) end def deserialize ( binary , config \\\\ [ ] ) do binary |&gt; decompress ( ) |&gt; Commanded.Serialization.JsonSerializer . deserialize ( config ) end defp compress ( term ) , do : ... defp decompress ( binary ) , do : ... end","ref":"serialization.html#customising-serialization","title":"Serialization - Customising serialization","type":"extras"},{"doc":"Your read model can be built using a Commanded event handler and whichever storage provider you prefer. You can choose to use a SQL or NoSQL database, document store, the filesystem, a full text search index, or any other storage mechanism. You may even use multiple storage providers, optimised for the querying they must support.","ref":"read-model-projections.html","title":"Read model projections","type":"extras"},{"doc":"You can use the Commanded Ecto projections library to build a read model using one of the databases supported by Ecto (PostgreSQL, MySQL, et al). Example defmodule MyApp.ExampleProjector do use Commanded.Projections.Ecto , application : MyApp.ExampleApp , name : &quot;ExampleProjector&quot; project % AnEvent { name : name } , _metadata do Ecto.Multi . insert ( multi , :example_projection , % ExampleProjection { name : name } ) end project % AnotherEvent { name : name } do Ecto.Multi . insert ( multi , :example_projection , % ExampleProjection { name : name } ) end end","ref":"read-model-projections.html#ecto-projections","title":"Read model projections - Ecto projections","type":"extras"},{"doc":"You will often choose to use :strong consistency for read model projections to ensure that you can query data affected by a dispatched command. In a typical web request using the POST/Redirect/GET pattern you want to ensure the read model is up-to-date before redirecting the user to the modified resource. By opting in to strong consistency you are guaranteed that an :ok reply from command dispatch indicates all strongly consistent read models will have been updated. Configure the consistency option in your projector: defmodule MyApp.ExampleProjector do use Commanded.Projections.Ecto , application : MyApp.ExampleApp , name : &quot;ExampleProjector&quot; , consistency : :strong end","ref":"read-model-projections.html#consistency-guarantee","title":"Read model projections - Consistency guarantee","type":"extras"},{"doc":"When using ES/CQRS, events are first-class citizens. It's critical to be able to assert that specific events are being emitted. Commanded provides test helpers to simplify your life.","ref":"testing.html","title":"Testing","type":"extras"},{"doc":"Please refer to the Testing your application page on the Wiki for help with configuring your test environment. Remember projection_versions when truncating tables If you rely on your read projections in your tests, remember to truncate the projection_versions table in your truncate_readstore_tables/0 function. Otherwise, your projector will ignore everything but the first projection. defp truncate_readstore_tables do &quot;&quot;&quot; TRUNCATE TABLE table1, table2, table3, projection_versions RESTART IDENTITY CASCADE; &quot;&quot;&quot; end","ref":"testing.html#setting-up-your-test-environment-your-tests","title":"Testing - Setting up your test environment / your tests","type":"extras"},{"doc":"Often you'll want to make sure a given event is published. Commanded provides assert_receive_event/3 and assert_receive_event/4 functions in the Commanded.Assertions.EventAssertions module to help with this. In the first case, we check that any event is received and use the argument as an assertion. In the second case, we also provide a predicate function that we can use to narrow our search down to a specific event. import Commanded.Assertions.EventAssertions test &quot;ensure any event of this type is published&quot; do :ok = MyApp . dispatch ( % Command { id : 4 , date : Date . today } ) assert_receive_event ( MyApp , Event , fn event -&gt; assert event . id == 4 end ) end test &quot;ensure an event is published matching the given predicate&quot; do :ok = MyApp . dispatch ( % Command { id : 4 , date : Date . today } ) assert_receive_event ( MyApp , Event , fn event -&gt; event . id == 4 end , fn event -&gt; assert event . date == Date . today end ) end","ref":"testing.html#asserting-that-an-event-is-published","title":"Testing - Asserting that an event is published","type":"extras"},{"doc":"Use the wait_for_event/2 and wait_for_event/3 functions to pause until a specific type of event, or event type matching a given predicate, is received. This can help you deal with eventual consistency in your tests. import Commanded.Assertions.EventAssertions test &quot;pause until specific event is published&quot; do :ok = BankApp . dispatch ( % OpenBankAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } ) wait_for_event ( BankApp , BankAccountOpened , fn opened -&gt; opened . account_number == &quot;ACC123&quot; end ) end","ref":"testing.html#waiting-for-an-event","title":"Testing - Waiting for an event","type":"extras"},{"doc":"It's a given that when going through CQRS, sometimes many events are part of the same action, either because they are returned from the aggregate together, or because event handlers trigger new commands which generate new events, etc. We will usually want to know, for audit trail purposes, that these events belong together. For this purpose, Commanded provides assert_correlated/4 which can be used to ensure that specific events have the same correlation_id : import Commanded.Assertions.EventAssertions test &quot;make sure two events are correlated&quot; do :ok = BankApp . dispatch ( % OpenBankAccount { account_number : &quot;ACC123&quot; , initial_balance : 1_000 } ) assert_correlated ( BankApp , BankAccountOpened , fn opened -&gt; opened . account_number == &quot;ACC123&quot; end , InitialAmountDeposited , fn deposited -&gt; deposited . account_number == &quot;ACC123&quot; end ) end","ref":"testing.html#correlation","title":"Testing - Correlation","type":"extras"},{"doc":"Sometimes it's useful to compare an expected aggregate's state with the previous one. This kind of function should be used only for testing. For this purpose, Commanded provides an aggregate_state function which returns the current aggregate state. import Commanded.Assertions.EventAssertions alias Commanded.Aggregates.Aggregate test &quot;make sure aggregate state are what we wanted&quot; do account_number = &quot;ACC123&quot; :ok = BankApp . dispatch ( % OpenBankAccount { account_number : account_number , initial_balance : 1_000 } ) :ok = BankApp . dispatch ( % WithdrawnMoney { account_number : account_number , amount : 200 } ) wait_for_event ( BankApp , BankAccountOpened , fn opened -&gt; opened . account_number == &quot;ACC123&quot; end ) wait_for_event ( BankApp , MoneyWithdrawn , fn withdrawn -&gt; withdrawn . balance == 800 end ) assert Aggregate . aggregate_state ( BankApp , BankAccount , account_number ) == % BankAccount { account_number : account_number , balance : 800 , state : :active } end","ref":"testing.html#aggregate-state-testing","title":"Testing - Aggregate state testing","type":"extras"},{"doc":"Commanded supports running on a single node, or multiple nodes run as either a distributed Erlang cluster or as multiple single instance nodes.","ref":"deployment.html","title":"Deployment","type":"extras"},{"doc":"Running your app using Commanded on a single node requires no configuration as local is the default setting.","ref":"deployment.html#single-node-deployment","title":"Deployment - Single node deployment","type":"extras"},{"doc":"To support deployment to a cluster of nodes and distributed Erlang you must configure: A registry which supports distributed Erlang. The :global registry provided by Commanded or the Commanded Swarm registry library. Phoenix's distributed pub/sub and presence platform to allow process distribution and communication amongst all nodes in the cluster. :global registry Use Erlang's :global name registration facility with distributed Erlang. The global name server starts automatically when a node is started. The registered names are stored in replicated global name tables on every node. Thus, the translation of a name to a pid is fast, as it is always done locally. Define the :global registry for your application: defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app , event_store : [ adapter : Commanded.EventStore.Adapters.EventStore , event_store : MyApp.EventStore ] , registry : :global end Or configure your application to use the :global registry in config: # config/config.exs config :my_app , MyApp.Application , registry : :global Note that when clusters are formed dynamically (e.g. using libcluster ]), the typical sequence of events is that first all nodes will start all processes, then the cluster is formed and :global will kill off duplicate names. This is ugly in the logs but expected; it also means that if your supervisor's :max_restarts is too low - lower than the number of event handlers/projectors you start - it will immediately exit and if that was your application supervisor, your app gets shutdown. The solution is simple: keep :max_restarts above the number of event handlers you start under your supervisor and the transition from no cluster to cluster will be clean. Commanded Swarm registry Add commanded_swarm_registry to your list of dependencies in mix.exs : def deps do [ { :commanded_swarm_registry , &quot;~&gt; 1.0&quot; } ] end Fetch mix dependencies: $ mix deps.get Configure your application to use the Swarm registry: # config/config.exs config :my_app , MyApp.Application , registry : Commanded.Registration.SwarmRegistry This uses the Swarm to distribute processes amongst the available nodes in the cluster. Phoenix pub/sub First, add it as a dependency to your project's mix.exs file: defp deps do [ { :phoenix_pubsub , &quot;~&gt; 2.0&quot; } ] end Fetch mix dependencies and configure the pubsub settings for your application in the environment config file: # config/config.exs config :my_app , MyApp.Application , pubsub : [ phoenix_pubsub : [ adapter : Phoenix.PubSub.PG2 , pool_size : 1 ] ] The PG2 adapter is preferable for cluster deployment since it is provided by Erlang and requires no further dependencies. EventStore If using PostgreSQL-based Elixir EventStore please also refer to its documentation about running on a clustering of nodes .","ref":"deployment.html#multi-node-distributed-erlang-deployment","title":"Deployment - Multi node distributed Erlang deployment","type":"extras"},{"doc":"Running multiple nodes, but choosing not to connect the nodes together to form a distributed Erlang cluster, requires that you use the local registry and Phoenix's pub/sub library with its Redis adapter. You must install and use Phoenix's pub/sub library, as described above . Since the nodes aren't connected, you are required to use the Redis adapter as a way of communicating between the nodes. Therefore you will need to host a Redis instance for use by your app. # config/config.exs config :my_app , MyApp.Application , registry : :local , pubsub : [ phoenix_pubsub : [ adapter : Phoenix.PubSub.Redis , host : &quot;localhost&quot; , port : 6379 , node_name : &quot;localhost&quot; ] ]","ref":"deployment.html#multi-node-but-not-distributed-erlang-deployment","title":"Deployment - Multi-node, but not distributed Erlang deployment","type":"extras"},{"doc":"In v0.19.x and earlier a singleton Commanded application was used to host aggregates and support event handlers, process managers, and other infrastructure processes. For v1.0, support for multiple Commanded apps was added allowing you to define and use more than one Commanded app and to control its lifecycle. Follow the following upgrade advice to migrate your application to Commanded v1.0. Commanded application First you must define a Commanded application: defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app router ( MyApp.Router ) end The application needs to be configured. You can do so in application config: # config/config.exs config :my_app , MyApp.Application , event_store : [ adapter : Commanded.EventStore.Adapters.EventStore , event_store : MyApp.EventStore ] , pubsub : :local , registry : :local Alternatively, you can include the event store, pubsub, and registry config when defining the application: defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app , event_store : [ adapter : Commanded.EventStore.Adapters.EventStore , event_store : MyApp.EventStore ] , pubsub : :local , registry : :local router ( MyApp.Router ) end Event store configuration To use Commanded v1.0 you must also upgrade the adapter for the event store you are using. Postgres EventStore Use the commanded_eventstore_adapter and follow the Getting started guide . In brief, the upgrade requires you to: Define an event store module for your Commanded application: defmodule MyApp.EventStore do use EventStore , otp_app : :my_app end Add a config entry containing the PostgreSQL database connection details for your event store module to each environment's mix config file (e.g. config/dev.exs ): config :my_app , MyApp.EventStore , serializer : Commanded.Serialization.JsonSerializer , username : &quot;postgres&quot; , password : &quot;postgres&quot; , database : &quot;eventstore&quot; , hostname : &quot;localhost&quot; Configure the Commanded application to use the event store: defmodule MyApp.Application do use Commanded.Application , otp_app : :my_app , event_store : [ adapter : Commanded.EventStore.Adapters.EventStore , event_store : MyApp.EventStore ] end Event Store Use the commanded_extreme_adapter and follow the Getting started guide . Supervision The Commanded application must be included and started somewhere in your app's supervision tree. You can include it within the top level application supervisor: defmodule MyApp do use Application def start ( _type , _args ) do children = [ MyApp.Application ] opts = [ strategy : :one_for_one , name : MyApp.Supervisor ] Supervisor . start_link ( children , opts ) end end Command dispatch Previously commands were always dispatched via a router module. Now, you dispatch commands using your Commanded application: :ok = MyApp.Application . dispatch ( % DoSomething { .. } ) The above should be a straightforward find within your source code to replace the router module (e.g. MyApp.Router ) with the new application module (e.g. MyApp.Application ). Optionally, you can dispatch a command using an existing router by configuring the router with the Commanded application: defmodule MyApp.Router do use Commanded.Commands.Router , application : MyApp.Application end :ok = MyApp.Router . dispatch ( % DoSomething { .. } ) It is also possible to specify the application during command dispatch: :ok = MyApp.Router . dispatch ( % DoSomething { .. } , application : MyApp.Application ) Event handlers and process managers Each handler, read model projections, and process manager needs to be provided with the additional application option: defmodule MyApp.ExampleHandler do use Commanded.Event.Handler , application : MyApp.Application , name : &quot;MyApp.ExampleHandler&quot; end Upgrade Commanded supporting libraries You will need to upgrade to the v1.0 versions of any Commanded related library used by your application, such as the event store adapter and Ecto read model projections.","ref":"0.19-1.0.html","title":"Upgrade guide v0.19.x to v1.0","type":"extras"}]